#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/4/21 15:34
Desc: 高频数据-标普 500 指数
https://github.com/FutureSharks/financial-data
long history data for S&P 500 index daily
http://www.econ.yale.edu/~shiller/data.htm
"""

import pandas as pd


def hf_sp_500(year: str = "2017") -> pd.DataFrame:
    """
    S&P 500 minute data from 2012-2018
    :param year: from 2012-2018
    :type year: str
    :return: specific year dataframe
    :rtype: pandas.DataFrame
    """
    url = f"https://github.com/FutureSharks/financial-data/raw/master/pyfinancialdata/data/stocks/histdata/SPXUSD/DAT_ASCII_SPXUSD_M1_{year}.csv"
    temp_df = pd.read_table(url, header=None, sep=";")
    temp_df.columns = ["date", "open", "high", "low", "close", "price"]
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"])
    temp_df["high"] = pd.to_numeric(temp_df["high"])
    temp_df["low"] = pd.to_numeric(temp_df["low"])
    temp_df["close"] = pd.to_numeric(temp_df["close"])
    temp_df["price"] = pd.to_numeric(temp_df["price"])
    return temp_df


if __name__ == "__main__":
    hf_sp_500_df = hf_sp_500(year="2017")
    print(hf_sp_500_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/6/19 22:00
Desc: 巨潮资讯-数据中心-专题统计-债券报表-债券发行
http://webapi.cninfo.com.cn/#/thematicStatistics
"""

import pandas as pd
import requests
import py_mini_racer

from akshare.datasets import get_ths_js


def _get_file_content_cninfo(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def bond_treasure_issue_cninfo(
    start_date: str = "20210910", end_date: str = "20211109"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-国债发行
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 结束统计数据
    :type end_date: str
    :return: 国债发行
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1120"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F009D": "缴款日",
            "SECNAME": "债券简称",
            "DECLAREDATE": "公告日期",
            "F004D": "发行起始日",
            "F003D": "发行终止日",
            "F008N": "单位面值",
            "SECCODE": "债券代码",
            "F007N": "发行价格",
            "F006N": "计划发行总量",
            "F005N": "实际发行总量",
            "F028N": "增发次数",
            "BONDNAME": "债券名称",
            "F014V": "发行对象",
            "F002V": "交易市场",
            "F013V": "发行方式",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "债券代码",
            "债券简称",
            "发行起始日",
            "发行终止日",
            "计划发行总量",
            "实际发行总量",
            "发行价格",
            "单位面值",
            "缴款日",
            "增发次数",
            "交易市场",
            "发行方式",
            "发行对象",
            "公告日期",
            "债券名称",
        ]
    ]
    temp_df["发行起始日"] = pd.to_datetime(
        temp_df["发行起始日"], errors="coerce"
    ).dt.date
    temp_df["发行终止日"] = pd.to_datetime(
        temp_df["发行终止日"], errors="coerce"
    ).dt.date
    temp_df["缴款日"] = pd.to_datetime(temp_df["缴款日"], errors="coerce").dt.date
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"], errors="coerce").dt.date
    temp_df["计划发行总量"] = pd.to_numeric(temp_df["计划发行总量"], errors="coerce")
    temp_df["实际发行总量"] = pd.to_numeric(temp_df["实际发行总量"], errors="coerce")
    temp_df["发行价格"] = pd.to_numeric(temp_df["发行价格"], errors="coerce")
    temp_df["单位面值"] = pd.to_numeric(temp_df["单位面值"], errors="coerce")
    temp_df["增发次数"] = pd.to_numeric(temp_df["增发次数"], errors="coerce")
    return temp_df


def bond_local_government_issue_cninfo(
    start_date: str = "20210911", end_date: str = "20211110"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-地方债发行
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 开始统计时间
    :type end_date: str
    :return: 地方债发行
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1121"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F009D": "缴款日",
            "SECNAME": "债券简称",
            "DECLAREDATE": "公告日期",
            "F004D": "发行起始日",
            "F003D": "发行终止日",
            "F008N": "单位面值",
            "SECCODE": "债券代码",
            "F007N": "发行价格",
            "F006N": "计划发行总量",
            "F005N": "实际发行总量",
            "F028N": "增发次数",
            "BONDNAME": "债券名称",
            "F014V": "发行对象",
            "F002V": "交易市场",
            "F013V": "发行方式",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "债券代码",
            "债券简称",
            "发行起始日",
            "发行终止日",
            "计划发行总量",
            "实际发行总量",
            "发行价格",
            "单位面值",
            "缴款日",
            "增发次数",
            "交易市场",
            "发行方式",
            "发行对象",
            "公告日期",
            "债券名称",
        ]
    ]
    temp_df["发行起始日"] = pd.to_datetime(
        temp_df["发行起始日"], errors="coerce"
    ).dt.date
    temp_df["发行终止日"] = pd.to_datetime(
        temp_df["发行终止日"], errors="coerce"
    ).dt.date
    temp_df["缴款日"] = pd.to_datetime(temp_df["缴款日"], errors="coerce").dt.date
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"], errors="coerce").dt.date
    temp_df["计划发行总量"] = pd.to_numeric(temp_df["计划发行总量"], errors="coerce")
    temp_df["实际发行总量"] = pd.to_numeric(temp_df["实际发行总量"], errors="coerce")
    temp_df["发行价格"] = pd.to_numeric(temp_df["发行价格"], errors="coerce")
    temp_df["单位面值"] = pd.to_numeric(temp_df["单位面值"], errors="coerce")
    temp_df["增发次数"] = pd.to_numeric(temp_df["增发次数"], errors="coerce")
    return temp_df


def bond_corporate_issue_cninfo(
    start_date: str = "20210911", end_date: str = "20211110"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-企业债发行
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 开始统计时间
    :type end_date: str
    :return: 企业债发行
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1122"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "SECNAME": "债券简称",
            "DECLAREDATE": "公告日期",
            "F004D": "交易所网上发行终止日",
            "F003D": "交易所网上发行起始日",
            "F008N": "发行面值",
            "SECCODE": "债券代码",
            "F007N": "发行价格",
            "F006N": "实际发行总量",
            "F005N": "计划发行总量",
            "F022N": "最小认购单位",
            "F017V": "承销方式",
            "F052N": "最低认购额",
            "F015V": "发行范围",
            "BONDNAME": "债券名称",
            "F014V": "发行对象",
            "F013V": "发行方式",
            "F023V": "募资用途说明",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "债券代码",
            "债券简称",
            "公告日期",
            "交易所网上发行起始日",
            "交易所网上发行终止日",
            "计划发行总量",
            "实际发行总量",
            "发行面值",
            "发行价格",
            "发行方式",
            "发行对象",
            "发行范围",
            "承销方式",
            "最小认购单位",
            "募资用途说明",
            "最低认购额",
            "债券名称",
        ]
    ]
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"], errors="coerce").dt.date
    temp_df["交易所网上发行起始日"] = pd.to_datetime(
        temp_df["交易所网上发行起始日"], errors="coerce"
    ).dt.date
    temp_df["交易所网上发行终止日"] = pd.to_datetime(
        temp_df["交易所网上发行终止日"], errors="coerce"
    ).dt.date
    temp_df["计划发行总量"] = pd.to_numeric(temp_df["计划发行总量"], errors="coerce")
    temp_df["实际发行总量"] = pd.to_numeric(temp_df["实际发行总量"], errors="coerce")
    temp_df["发行面值"] = pd.to_numeric(temp_df["发行面值"], errors="coerce")
    temp_df["发行价格"] = pd.to_numeric(temp_df["发行价格"], errors="coerce")
    temp_df["最小认购单位"] = pd.to_numeric(temp_df["最小认购单位"], errors="coerce")
    temp_df["最低认购额"] = pd.to_numeric(temp_df["最低认购额"], errors="coerce")
    return temp_df


def bond_cov_issue_cninfo(
    start_date: str = "20210913", end_date: str = "20211112"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-可转债发行
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 开始统计时间
    :type end_date: str
    :return: 可转债发行
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1123"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F029D": "发行起始日",
            "SECNAME": "债券简称",
            "F027D": "转股开始日期",
            "F003D": "发行终止日",
            "F007N": "发行面值",
            "F053D": "转股终止日期",
            "F005N": "计划发行总量",
            "F051D": "网上申购日期",
            "F026N": "初始转股价格",
            "F066N": "网上申购数量下限",
            "F052N": "发行价格",
            "BONDNAME": "债券名称",
            "F014V": "发行对象",
            "F002V": "交易市场",
            "F032V": "网上申购简称",
            "F086V": "转股代码",
            "DECLAREDATE": "公告日期",
            "F028D": "债权登记日",
            "F004D": "优先申购日",
            "F068D": "网上申购中签结果公告日及退款日",
            "F054D": "优先申购缴款日",
            "F008N": "网上申购数量上限",
            "SECCODE": "债券代码",
            "F006N": "实际发行总量",
            "F067N": "网上申购单位",
            "F065N": "配售价格",
            "F017V": "承销方式",
            "F015V": "发行范围",
            "F013V": "发行方式",
            "F021V": "募资用途说明",
            "F031V": "网上申购代码",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "债券代码",
            "债券简称",
            "公告日期",
            "发行起始日",
            "发行终止日",
            "计划发行总量",
            "实际发行总量",
            "发行面值",
            "发行价格",
            "发行方式",
            "发行对象",
            "发行范围",
            "承销方式",
            "募资用途说明",
            "初始转股价格",
            "转股开始日期",
            "转股终止日期",
            "网上申购日期",
            "网上申购代码",
            "网上申购简称",
            "网上申购数量上限",
            "网上申购数量下限",
            "网上申购单位",
            "网上申购中签结果公告日及退款日",
            "优先申购日",
            "配售价格",
            "债权登记日",
            "优先申购缴款日",
            "转股代码",
            "交易市场",
            "债券名称",
        ]
    ]
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"], errors="coerce").dt.date
    temp_df["发行起始日"] = pd.to_datetime(
        temp_df["发行起始日"], errors="coerce"
    ).dt.date
    temp_df["发行终止日"] = pd.to_datetime(
        temp_df["发行终止日"], errors="coerce"
    ).dt.date
    temp_df["转股开始日期"] = pd.to_datetime(
        temp_df["转股开始日期"], errors="coerce"
    ).dt.date
    temp_df["转股终止日期"] = pd.to_datetime(
        temp_df["转股终止日期"], errors="coerce"
    ).dt.date
    temp_df["转股终止日期"] = pd.to_datetime(
        temp_df["转股终止日期"], errors="coerce"
    ).dt.date
    temp_df["网上申购日期"] = pd.to_datetime(
        temp_df["网上申购日期"], errors="coerce"
    ).dt.date
    temp_df["网上申购中签结果公告日及退款日"] = pd.to_datetime(
        temp_df["网上申购中签结果公告日及退款日"], errors="coerce"
    ).dt.date
    temp_df["债权登记日"] = pd.to_datetime(
        temp_df["债权登记日"], errors="coerce"
    ).dt.date
    temp_df["优先申购日"] = pd.to_datetime(
        temp_df["优先申购日"], errors="coerce"
    ).dt.date
    temp_df["优先申购缴款日"] = pd.to_datetime(
        temp_df["优先申购缴款日"], errors="coerce"
    ).dt.date
    temp_df["计划发行总量"] = pd.to_numeric(temp_df["计划发行总量"], errors="coerce")
    temp_df["实际发行总量"] = pd.to_numeric(temp_df["实际发行总量"], errors="coerce")
    temp_df["发行面值"] = pd.to_numeric(temp_df["发行面值"], errors="coerce")
    temp_df["发行价格"] = pd.to_numeric(temp_df["发行价格"], errors="coerce")
    temp_df["初始转股价格"] = pd.to_numeric(temp_df["初始转股价格"], errors="coerce")
    temp_df["网上申购数量上限"] = pd.to_numeric(
        temp_df["网上申购数量上限"], errors="coerce"
    )
    temp_df["网上申购数量下限"] = pd.to_numeric(
        temp_df["网上申购数量下限"], errors="coerce"
    )
    temp_df["网上申购单位"] = pd.to_numeric(temp_df["网上申购单位"], errors="coerce")
    temp_df["配售价格"] = pd.to_numeric(temp_df["配售价格"], errors="coerce")
    return temp_df


def bond_cov_stock_issue_cninfo() -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-可转债转股
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :return: 可转债转股
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1124"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F003N": "转股价格",
            "SECNAME": "债券简称",
            "DECLAREDATE": "公告日期",
            "F005D": "自愿转换期终止日",
            "F004D": "自愿转换期起始日",
            "F017V": "标的股票",
            "BONDNAME": "债券名称",
            "F002V": "转股简称",
            "F001V": "转股代码",
            "SECCODE": "债券代码",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "债券代码",
            "债券简称",
            "公告日期",
            "转股代码",
            "转股简称",
            "转股价格",
            "自愿转换期起始日",
            "自愿转换期终止日",
            "标的股票",
            "债券名称",
        ]
    ]
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"], errors="coerce").dt.date
    temp_df["自愿转换期起始日"] = pd.to_datetime(
        temp_df["自愿转换期起始日"], errors="coerce"
    ).dt.date
    temp_df["自愿转换期终止日"] = pd.to_datetime(
        temp_df["自愿转换期终止日"], errors="coerce"
    ).dt.date
    temp_df["转股价格"] = pd.to_numeric(temp_df["转股价格"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    bond_treasure_issue_cninfo_df = bond_treasure_issue_cninfo(
        start_date="20210910", end_date="20211109"
    )
    print(bond_treasure_issue_cninfo_df)

    bond_local_government_issue_cninfo_df = bond_local_government_issue_cninfo(
        start_date="20210911", end_date="20211110"
    )
    print(bond_local_government_issue_cninfo_df)

    bond_corporate_issue_cninfo_df = bond_corporate_issue_cninfo(
        start_date="20210911", end_date="20211110"
    )
    print(bond_corporate_issue_cninfo_df)

    bond_cov_issue_cninfo_df = bond_cov_issue_cninfo(
        start_date="20210913", end_date="20211112"
    )
    print(bond_cov_issue_cninfo_df)

    bond_cov_stock_issue_cninfo_df = bond_cov_stock_issue_cninfo()
    print(bond_cov_stock_issue_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/1/20 17:00
Desc: 东方财富网站-天天基金网-基金数据-开放式基金净值
https://fund.eastmoney.com/manager/default.html#dt14;mcreturnjson;ftall;pn20;pi1;scabbname;stasc
1.基金经理基本数据, 建议包含:基金经理代码,基金经理姓名,从业起始日期,现任基金公司,管理资产总规模,上述数据可在"基金经理列表:
https://fund.eastmoney.com/manager/default.html#dt14;mcreturnjson;ftall;pn20;pi1;scabbname;stasc 和
"基金经理理档案如:https://fund.eastmoney.com/manager/30040164.html 获取.
2.基金经理任职数据:可调取全部或特定经理,管理的基金数据,建议包含:基金经理代码,基金经理姓名,基金代码,基金简称,
经理位次(在当前基金的经理中排第几位),起始任职时间,截止任职时间,任职回报.在特定基金的经理信息中可以获取如:
https://fundf10.eastmoney.com/jjjl_001810.html
3.在接口：fund_basic"公募基金列表"增加数据"基金经理代码"(或第一基金经理代码),
"基金经理姓名"(或第一基金经理姓名),"当前基金经理人数","当前经理任职起始时间".
用户ID:269993
"""

import json
import math
import time
from io import StringIO

import pandas as pd
import py_mini_racer
import requests

from akshare.utils import demjson
from akshare.utils.cons import headers
from akshare.utils.tqdm import get_tqdm


def fund_purchase_em() -> pd.DataFrame:
    """
    东方财富网站-天天基金网-基金数据-基金申购状态
    https://fund.eastmoney.com/Fund_sgzt_bzdm.html#fcode,asc_1
    :return: 基金申购状态
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/Data/Fund_JJJZ_Data.aspx"
    params = {
        "t": "8",
        "page": "1,50000",
        "js": "reData",
        "sort": "fcode,asc",
    }
    r = requests.get(url, params=params, headers=headers)
    data_text = r.text
    data_json = demjson.decode(data_text.strip("var reData="))
    temp_df = pd.DataFrame(data_json["datas"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "基金代码",
        "基金简称",
        "基金类型",
        "最新净值/万份收益",
        "最新净值/万份收益-报告时间",
        "申购状态",
        "赎回状态",
        "下一开放日",
        "购买起点",
        "日累计限定金额",
        "-",
        "-",
        "手续费",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "基金类型",
            "最新净值/万份收益",
            "最新净值/万份收益-报告时间",
            "申购状态",
            "赎回状态",
            "下一开放日",
            "购买起点",
            "日累计限定金额",
            "手续费",
        ]
    ]
    temp_df["下一开放日"] = pd.to_datetime(
        temp_df["下一开放日"], errors="coerce"
    ).dt.date
    temp_df["最新净值/万份收益"] = pd.to_numeric(
        temp_df["最新净值/万份收益"], errors="coerce"
    )
    temp_df["购买起点"] = pd.to_numeric(temp_df["购买起点"], errors="coerce")
    temp_df["日累计限定金额"] = pd.to_numeric(
        temp_df["日累计限定金额"], errors="coerce"
    )
    temp_df["手续费"] = temp_df["手续费"].str.strip("%")
    temp_df["手续费"] = pd.to_numeric(temp_df["手续费"], errors="coerce")
    return temp_df


def fund_name_em() -> pd.DataFrame:
    """
    东方财富网站-天天基金网-基金数据-所有基金的名称和类型
    https://fund.eastmoney.com/manager/default.html#dt14;mcreturnjson;ftall;pn20;pi1;scabbname;stasc
    :return: 所有基金的名称和类型
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/js/fundcode_search.js"
    r = requests.get(url, headers=headers)
    text_data = r.text
    data_json = demjson.decode(text_data.strip("var r = ")[:-1])
    temp_df = pd.DataFrame(data_json)
    temp_df.columns = ["基金代码", "拼音缩写", "基金简称", "基金类型", "拼音全称"]
    return temp_df


def fund_info_index_em(
    symbol: str = "沪深指数", indicator: str = "被动指数型"
) -> pd.DataFrame:
    """
    东方财富网站-天天基金网-基金数据-基金信息-指数型
    https://fund.eastmoney.com/trade/zs.html
    :param symbol: choice of {"全部", "沪深指数", "行业主题", "大盘指数", "中盘指数", "小盘指数", "股票指数", "债券指数"}
    :type symbol: str
    :param indicator: choice of {"全部", "被动指数型", "增强指数型"}
    :type indicator: str
    :return: 基金信息-指数型
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "全部": "",
        "沪深指数": "053",
        "行业主题": "054",
        "大盘指数": "01",
        "中盘指数": "02",
        "小盘指数": "03",
        "股票指数": "050|001",
        "债券指数": "050|003",
    }
    indicator_map = {
        "全部": "",
        "被动指数型": "051",
        "增强指数型": "052",
    }
    url = "https://api.fund.eastmoney.com/FundTradeRank/GetRankList"
    if symbol in {"股票指数", "债券指数"}:
        params = {
            "ft": "zs",
            "sc": "1n",
            "st": "desc",
            "pi": "1",
            "pn": "10000",
            "cp": "",
            "ct": "",
            "cd": "",
            "ms": "",
            "fr": symbol_map[symbol].split("|")[0],
            "plevel": "",
            "fst": "",
            "ftype": symbol_map[symbol].split("|")[1],
            "fr1": indicator_map[indicator],
            "fl": "0",
            "isab": "1",
        }
    else:
        params = {
            "ft": "zs",
            "sc": "1n",
            "st": "desc",
            "pi": "1",
            "pn": "10000",
            "cp": "",
            "ct": "",
            "cd": "",
            "ms": "",
            "fr": symbol_map[symbol].split("|")[0],
            "plevel": "",
            "fst": "",
            "ftype": "",
            "fr1": indicator_map[indicator],
            "fl": "0",
            "isab": "1",
        }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Host": "api.fund.eastmoney.com",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://fund.eastmoney.com/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/103.0.0.0 Safari/537.36",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    data_json = json.loads(data_json["Data"])
    temp_df = pd.DataFrame([item.split("|") for item in data_json["datas"]])
    temp_df.columns = [
        "基金代码",
        "基金名称",
        "-",
        "日期",
        "单位净值",
        "日增长率",
        "近1周",
        "近1月",
        "近3月",
        "近6月",
        "近1年",
        "近2年",
        "近3年",
        "今年来",
        "成立来",
        "-",
        "-",
        "-",
        "手续费",
        "-",
        "-",
        "-",
        "-",
        "-",
        "起购金额",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "基金代码",
            "基金名称",
            "单位净值",
            "日期",
            "日增长率",
            "近1周",
            "近1月",
            "近3月",
            "近6月",
            "近1年",
            "近2年",
            "近3年",
            "今年来",
            "成立来",
            "手续费",
            "起购金额",
        ]
    ]
    temp_df["跟踪标的"] = symbol
    temp_df["跟踪方式"] = indicator

    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["日增长率"] = pd.to_numeric(temp_df["日增长率"], errors="coerce")
    temp_df["近1周"] = pd.to_numeric(temp_df["近1周"], errors="coerce")
    temp_df["近1月"] = pd.to_numeric(temp_df["近1月"], errors="coerce")
    temp_df["近3月"] = pd.to_numeric(temp_df["近3月"], errors="coerce")
    temp_df["近6月"] = pd.to_numeric(temp_df["近6月"], errors="coerce")
    temp_df["近1年"] = pd.to_numeric(temp_df["近1年"], errors="coerce")
    temp_df["近2年"] = pd.to_numeric(temp_df["近2年"], errors="coerce")
    temp_df["近3年"] = pd.to_numeric(temp_df["近3年"], errors="coerce")
    temp_df["今年来"] = pd.to_numeric(temp_df["今年来"], errors="coerce")
    temp_df["成立来"] = pd.to_numeric(temp_df["成立来"], errors="coerce")
    temp_df["手续费"] = pd.to_numeric(temp_df["手续费"], errors="coerce")
    return temp_df


def fund_open_fund_daily_em() -> pd.DataFrame:
    """
    东方财富网-天天基金网-基金数据-开放式基金净值
    https://fund.eastmoney.com/fund.html#os_0;isall_0;ft_;pt_1
    :return: 当前交易日的所有开放式基金净值数据
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/Data/Fund_JJJZ_Data.aspx"
    params = {
        "t": "1",
        "lx": "1",
        "letter": "",
        "gsid": "",
        "text": "",
        "sort": "zdf,desc",
        "page": "1,50000",
        "dt": "1580914040623",
        "atfc": "",
        "onlySale": "0",
    }
    res = requests.get(url, params=params, headers=headers)
    text_data = res.text
    data_json = demjson.decode(text_data.strip("var db="))
    temp_df = pd.DataFrame(data_json["datas"])
    show_day = data_json["showday"]
    temp_df.columns = [
        "基金代码",
        "基金简称",
        "-",
        f"{show_day[0]}-单位净值",
        f"{show_day[0]}-累计净值",
        f"{show_day[1]}-单位净值",
        f"{show_day[1]}-累计净值",
        "日增长值",
        "日增长率",
        "申购状态",
        "赎回状态",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "手续费",
        "-",
        "-",
        "-",
    ]
    data_df = temp_df[
        [
            "基金代码",
            "基金简称",
            f"{show_day[0]}-单位净值",
            f"{show_day[0]}-累计净值",
            f"{show_day[1]}-单位净值",
            f"{show_day[1]}-累计净值",
            "日增长值",
            "日增长率",
            "申购状态",
            "赎回状态",
            "手续费",
        ]
    ]
    return data_df


def fund_open_fund_info_em(
    symbol: str = "710001", indicator: str = "单位净值走势", period: str = "成立来"
) -> pd.DataFrame:
    """
    东方财富网-天天基金网-基金数据-开放式基金净值
    https://fund.eastmoney.com/fund.html
    :param symbol: 基金代码; 可以通过调用 ak.fund_open_fund_daily_em() 获取所有开放式基金代码
    :type symbol: str
    :param indicator: 需要获取的指标
    :type indicator: str
    :param period: "成立来"; choice of {"1月", "3月", "6月", "1年", "3年", "5年", "今年来", "成立来"}
    :type period: str
    :return: 指定基金指定指标的数据
    :rtype: pandas.DataFrame
    """
    from akshare.utils.cons import headers

    url = f"https://fund.eastmoney.com/pingzhongdata/{symbol}.js"  # 各类数据都在里面
    r = requests.get(url, headers=headers)
    data_text = r.text

    js_code = py_mini_racer.MiniRacer()
    js_code.eval(data_text)

    # 单位净值走势
    if indicator == "单位净值走势":
        data_json = js_code.execute("Data_netWorthTrend")
        temp_df = pd.DataFrame(data_json)
        if temp_df.empty:
            return pd.DataFrame()
        temp_df["x"] = pd.to_datetime(temp_df["x"], unit="ms", utc=True).dt.tz_convert(
            "Asia/Shanghai"
        )
        temp_df["x"] = temp_df["x"].dt.date
        temp_df.columns = [
            "净值日期",
            "单位净值",
            "日增长率",
            "_",
        ]
        temp_df = temp_df[
            [
                "净值日期",
                "单位净值",
                "日增长率",
            ]
        ]
        temp_df["净值日期"] = pd.to_datetime(
            temp_df["净值日期"], errors="coerce"
        ).dt.date
        temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
        temp_df["日增长率"] = pd.to_numeric(temp_df["日增长率"], errors="coerce")
        return temp_df

    # 累计净值走势
    if indicator == "累计净值走势":
        data_json = js_code.execute("Data_ACWorthTrend")
        temp_df = pd.DataFrame(data_json)
        if temp_df.empty:
            return pd.DataFrame()
        temp_df.columns = ["x", "y"]
        temp_df["x"] = pd.to_datetime(temp_df["x"], unit="ms", utc=True).dt.tz_convert(
            "Asia/Shanghai"
        )
        temp_df["x"] = temp_df["x"].dt.date
        temp_df.columns = [
            "净值日期",
            "累计净值",
        ]
        temp_df = temp_df[
            [
                "净值日期",
                "累计净值",
            ]
        ]
        temp_df["净值日期"] = pd.to_datetime(
            temp_df["净值日期"], errors="coerce"
        ).dt.date
        temp_df["累计净值"] = pd.to_numeric(temp_df["累计净值"], errors="coerce")
        return temp_df

    # 每万份收益
    if indicator == "每万份收益":
        data_json = js_code.execute("Data_millionCopiesIncome")
        temp_df = pd.DataFrame(data_json)
        if temp_df.empty:
            return pd.DataFrame()
        temp_df.columns = ["x", "y"]
        temp_df["x"] = pd.to_datetime(temp_df["x"], unit="ms", utc=True).dt.tz_convert(
            "Asia/Shanghai"
        )
        temp_df["x"] = temp_df["x"].dt.date
        temp_df.columns = [
            "净值日期",
            "每万份收益",
        ]
        temp_df = temp_df[
            [
                "净值日期",
                "每万份收益",
            ]
        ]
        temp_df["净值日期"] = pd.to_datetime(
            temp_df["净值日期"], errors="coerce"
        ).dt.date
        temp_df["每万份收益"] = pd.to_numeric(temp_df["每万份收益"], errors="coerce")
        return temp_df

    # 7日年化收益率
    if indicator == "7日年化收益率":
        data_json = js_code.execute("Data_sevenDaysYearIncome")
        temp_df = pd.DataFrame(data_json)
        if temp_df.empty:
            return pd.DataFrame()
        temp_df.columns = ["x", "y"]
        temp_df["x"] = pd.to_datetime(temp_df["x"], unit="ms", utc=True).dt.tz_convert(
            "Asia/Shanghai"
        )
        temp_df["x"] = temp_df["x"].dt.date
        temp_df.columns = [
            "净值日期",
            "7日年化收益率",
        ]
        temp_df = temp_df[
            [
                "净值日期",
                "7日年化收益率",
            ]
        ]
        temp_df["净值日期"] = pd.to_datetime(
            temp_df["净值日期"], errors="coerce"
        ).dt.date
        temp_df["7日年化收益率"] = pd.to_numeric(
            temp_df["7日年化收益率"], errors="coerce"
        )
        return temp_df

    # 累计收益率走势
    if indicator == "累计收益率走势":
        url = "https://api.fund.eastmoney.com/pinzhong/LJSYLZS"
        headers = {"Referer": "https://fund.eastmoney.com/"}
        period_map = {
            "1月": "m",
            "3月": "q",
            "6月": "hy",
            "1年": "y",
            "3年": "try",
            "5年": "fiy",
            "今年来": "sy",
            "成立来": "se",
        }
        params = {
            "fundCode": symbol,
            "indexcode": "000300",
            "type": period_map[period],
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["Data"][0]["data"])
        temp_df.columns = ["日期", "累计收益率"]
        temp_df["日期"] = pd.to_datetime(
            temp_df["日期"], unit="ms", utc=True
        ).dt.tz_convert("Asia/Shanghai")
        temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
        temp_df["累计收益率"] = pd.to_numeric(temp_df["累计收益率"], errors="coerce")
        return temp_df

    # 同类排名走势
    if indicator == "同类排名走势":
        data_json = js_code.execute("Data_rateInSimilarType")
        temp_df = pd.DataFrame(data_json)
        temp_df["x"] = pd.to_datetime(temp_df["x"], unit="ms", utc=True).dt.tz_convert(
            "Asia/Shanghai"
        )
        temp_df["x"] = temp_df["x"].dt.date
        temp_df.columns = [
            "报告日期",
            "同类型排名-每日近三月排名",
            "总排名-每日近三月排名",
        ]
        temp_df = temp_df[
            [
                "报告日期",
                "同类型排名-每日近三月排名",
                "总排名-每日近三月排名",
            ]
        ]
        temp_df["报告日期"] = pd.to_datetime(
            temp_df["报告日期"], errors="coerce"
        ).dt.date
        temp_df["同类型排名-每日近三月排名"] = pd.to_numeric(
            temp_df["同类型排名-每日近三月排名"], errors="coerce"
        )
        temp_df["总排名-每日近三月排名"] = pd.to_numeric(
            temp_df["总排名-每日近三月排名"], errors="coerce"
        )
        return temp_df

    # 同类排名百分比
    if indicator == "同类排名百分比":
        data_json = js_code.execute("Data_rateInSimilarPersent")
        temp_df = pd.DataFrame(data_json)
        temp_df.columns = ["x", "y"]
        temp_df["x"] = pd.to_datetime(temp_df["x"], unit="ms", utc=True).dt.tz_convert(
            "Asia/Shanghai"
        )
        temp_df["x"] = temp_df["x"].dt.date
        temp_df.columns = [
            "报告日期",
            "同类型排名-每日近3月收益排名百分比",
        ]
        temp_df = temp_df[
            [
                "报告日期",
                "同类型排名-每日近3月收益排名百分比",
            ]
        ]
        temp_df["报告日期"] = pd.to_datetime(
            temp_df["报告日期"], errors="coerce"
        ).dt.date
        temp_df["同类型排名-每日近3月收益排名百分比"] = pd.to_numeric(
            temp_df["同类型排名-每日近3月收益排名百分比"], errors="coerce"
        )
        return temp_df

    # 分红送配详情
    if indicator == "分红送配详情":
        url = f"https://fundf10.eastmoney.com/fhsp_{symbol}.html"
        r = requests.get(url, headers=headers)
        table_num = len(pd.read_html(StringIO(r.text)))
        if table_num == 3:
            temp_df = pd.read_html(StringIO(r.text))[1]
        else:
            temp_df = pd.read_html(StringIO(r.text))[0]
        if temp_df.iloc[0, 1] == "暂无分红信息!":
            return pd.DataFrame()
        else:
            return temp_df

    # 拆分详情
    if indicator == "拆分详情":
        url = f"https://fundf10.eastmoney.com/fhsp_{symbol}.html"
        r = requests.get(url, headers=headers)
        table_num = len(pd.read_html(StringIO(r.text)))
        if table_num == 3:
            temp_df = pd.read_html(StringIO(r.text))[2]
        else:
            temp_df = pd.read_html(StringIO(r.text))[1]
        if temp_df.iloc[0, 1] == "暂无拆分信息!":
            return pd.DataFrame()
        else:
            return temp_df
    return pd.DataFrame()


def fund_money_fund_daily_em() -> pd.DataFrame:
    """
    东方财富网-天天基金网-基金数据-货币型基金收益
    https://fund.eastmoney.com/HBJJ_pjsyl.html
    :return: 当前交易日的所有货币型基金收益数据
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/HBJJ_pjsyl.html"
    r = requests.get(url, headers=headers)
    r.encoding = "gb2312"
    show_day = pd.read_html(StringIO(r.text))[1].iloc[0, 5:11].tolist()
    temp_df = pd.read_html(StringIO(r.text))[1].iloc[1:, 2:]
    temp_df_columns = temp_df.iloc[0, :].tolist()[1:]
    temp_df = temp_df.iloc[1:, 1:]
    temp_df.columns = temp_df_columns
    temp_df["基金简称"] = temp_df["基金简称"].str.strip("基金吧档案")
    temp_df.columns = [
        "基金代码",
        "基金简称",
        f"{show_day[0]}-万份收益",
        f"{show_day[1]}-7日年化%",
        f"{show_day[2]}-单位净值",
        f"{show_day[3]}-万份收益",
        f"{show_day[4]}-7日年化%",
        f"{show_day[5]}-单位净值",
        "日涨幅",
        "成立日期",
        "基金经理",
        "手续费",
        "可购全部",
    ]
    return temp_df


def fund_money_fund_info_em(symbol: str = "000009") -> pd.DataFrame:
    """
    东方财富网-天天基金网-基金数据-货币型基金收益-历史净值数据
    https://fundf10.eastmoney.com/jjjz_004186.html
    :param symbol: 货币型基金代码, 可以通过 fund_money_fund_daily_em 来获取
    :type symbol: str
    :return: 东方财富网站-天天基金网-基金数据-货币型基金收益-历史净值数据
    :rtype: pandas.DataFrame
    """
    url = "https://api.fund.eastmoney.com/f10/lsjz"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36",
        "Referer": f"https://fundf10.eastmoney.com/jjjz_{symbol}.html",
        "Host": "api.fund.eastmoney.com",
    }
    params = {
        "fundCode": symbol,
        "pageIndex": "1",
        "pageSize": "20",
        "startDate": "",
        "endDate": "",
        "_": round(time.time() * 1000),
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    total_page = math.ceil(int(data_json["TotalCount"]) / 20)
    tqdm = get_tqdm()
    big_list = []
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageIndex": page})
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["Data"]["LSJZList"])
        big_list.append(temp_df)
    big_df = pd.concat(big_list, ignore_index=True)
    big_df.columns = [
        "净值日期",
        "每万份收益",
        "7日年化收益率",
        "_",
        "_",
        "_",
        "_",
        "申购状态",
        "赎回状态",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df.sort_values(by=["净值日期"], inplace=True, ignore_index=True)
    big_df = big_df[["净值日期", "每万份收益", "7日年化收益率", "申购状态", "赎回状态"]]
    big_df["净值日期"] = pd.to_datetime(big_df["净值日期"], errors="coerce").dt.date
    big_df["每万份收益"] = pd.to_numeric(big_df["每万份收益"], errors="coerce")
    big_df["7日年化收益率"] = pd.to_numeric(big_df["7日年化收益率"], errors="coerce")
    return big_df


def fund_financial_fund_daily_em() -> pd.DataFrame:
    """
    东方财富网站-天天基金网-基金数据-理财型基金收益
    # 该接口暂无数据
    https://fund.eastmoney.com/lcjj.html#1_1__0__ljjz,desc_1_os1
    :return: 当前交易日的所有理财型基金收益
    :rtype: pandas.DataFrame
    """
    url = "https://api.fund.eastmoney.com/FundNetValue/GetLCJJJZ"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36",
        "Referer": "https://fund.eastmoney.com/lcjj.html",
    }
    params = {
        "letter": "",
        "jjgsid": "0",
        "searchtext": "",
        "sort": "ljjz,desc",
        "page": "1,100",
        "AttentionCodes": "",
        "cycle": "",
        "OnlySale": "1",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["Data"]["List"])
    if temp_df.empty:
        return pd.DataFrame()
    show_day = data_json["Data"]["showday"]
    data_df = temp_df[
        [
            "Id",
            "actualsyi",
            "cycle",
            "fcode",
            "kfr",
            "mui",
            "shortname",
            "syi",
            "zrmui",
            "zrsyi",
        ]
    ]
    data_df.columns = [
        "序号",
        "上一期年化收益率",
        "封闭期",
        "基金代码",
        "申购状态",
        f"{show_day[0]}-万份收益",
        "基金简称",
        f"{show_day[0]}-7日年华",
        f"{show_day[1]}-万份收益",
        f"{show_day[1]}-7日年华",
    ]
    data_df = data_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "上一期年化收益率",
            f"{show_day[0]}-万份收益",
            f"{show_day[0]}-7日年华",
            f"{show_day[1]}-万份收益",
            f"{show_day[1]}-7日年华",
            "封闭期",
            "申购状态",
        ]
    ]
    return data_df


def fund_financial_fund_info_em(symbol: str = "000134") -> pd.DataFrame:
    """
    东方财富网站-天天基金网-基金数据-理财型基金收益-历史净值明细
    https://fundf10.eastmoney.com/jjjz_000791.html
    :param symbol: 理财型基金代码, 可以通过 ak.fund_financial_fund_daily_em() 来获取
    :type symbol: str
    :return: 东方财富网站-天天基金网-基金数据-理财型基金收益-历史净值明细
    :rtype: pandas.DataFrame
    """
    url = "https://api.fund.eastmoney.com/f10/lsjz"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36",
        "Referer": f"https://fundf10.eastmoney.com/jjjz_{symbol}.html",
    }
    params = {
        "callback": "jQuery18307915911837995662_1588249228826",
        "fundCode": symbol,
        "pageIndex": "1",
        "pageSize": "10000",
        "startDate": "",
        "endDate": "",
        "_": round(time.time() * 1000),
    }
    r = requests.get(url, params=params, headers=headers)
    text_data = r.text
    data_json = demjson.decode(text_data[text_data.find("{") : -1])
    temp_df = pd.DataFrame(data_json["Data"]["LSJZList"])
    temp_df.columns = [
        "净值日期",
        "单位净值",
        "累计净值",
        "_",
        "_",
        "_",
        "日增长率",
        "申购状态",
        "赎回状态",
        "_",
        "_",
        "_",
        "分红送配",
    ]
    temp_df = temp_df[
        [
            "净值日期",
            "单位净值",
            "累计净值",
            "日增长率",
            "申购状态",
            "赎回状态",
            "分红送配",
        ]
    ]
    temp_df.sort_values(["净值日期"], inplace=True, ignore_index=True)
    temp_df["净值日期"] = pd.to_datetime(temp_df["净值日期"]).dt.date
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["累计净值"] = pd.to_numeric(temp_df["累计净值"], errors="coerce")
    temp_df["日增长率"] = pd.to_numeric(temp_df["日增长率"], errors="coerce")
    return temp_df


def fund_graded_fund_daily_em() -> pd.DataFrame:
    """
    东方财富网站-天天基金网-基金数据-分级基金净值
    https://fund.eastmoney.com/fjjj.html#1_1__0__zdf,desc_1
    :return: 当前交易日的所有分级基金净值
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/Data/Fund_JJJZ_Data.aspx"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36",
        "Referer": "https://fund.eastmoney.com/fjjj.html",
    }
    params = {
        "t": "1",
        "lx": "9",
        "letter": "",
        "gsid": "0",
        "text": "",
        "sort": "zdf,desc",
        "page": "1,10000",
        "dt": "1580914040623",
        "atfc": "",
    }
    res = requests.get(url, params=params, headers=headers)
    text_data = res.text
    data_json = demjson.decode(text_data.strip("var db="))
    temp_df = pd.DataFrame(data_json["datas"])
    show_day = data_json["showday"]
    temp_df.columns = [
        "基金代码",
        "基金简称",
        "-",
        f"{show_day[0]}-单位净值",
        f"{show_day[0]}-累计净值",
        f"{show_day[1]}--单位净值",
        f"{show_day[1]}--累计净值",
        "日增长值",
        "日增长率",
        "市价",
        "折价率",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "手续费",
    ]
    data_df = temp_df[
        [
            "基金代码",
            "基金简称",
            f"{show_day[0]}-单位净值",
            f"{show_day[0]}-累计净值",
            f"{show_day[1]}--单位净值",
            f"{show_day[1]}--累计净值",
            "日增长值",
            "日增长率",
            "市价",
            "折价率",
            "手续费",
        ]
    ]
    return data_df


def fund_graded_fund_info_em(symbol: str = "150232") -> pd.DataFrame:
    """
    东方财富网站-天天基金网-基金数据-分级基金净值-历史净值明细
    https://fundf10.eastmoney.com/jjjz_150232.html
    :param symbol: 分级基金代码, 可以通过 ak.fund_money_fund_daily_em() 来获取
    :type symbol: str
    :return: 东方财富网站-天天基金网-基金数据-分级基金净值-历史净值明细
    :rtype: pandas.DataFrame
    """
    url = "https://api.fund.eastmoney.com/f10/lsjz"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36",
        "Referer": f"https://fundf10.eastmoney.com/jjjz_{symbol}.html",
    }
    params = {
        "fundCode": symbol,
        "pageIndex": "1",
        "pageSize": "20",
        "startDate": "",
        "endDate": "",
        "_": round(time.time() * 1000),
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    total_page = math.ceil(int(data_json["TotalCount"]) / 20)
    tqdm = get_tqdm()
    big_list = []
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageIndex": page})
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["Data"]["LSJZList"])
        big_list.append(temp_df)
    big_df = pd.concat(big_list, ignore_index=True)
    big_df.columns = [
        "净值日期",
        "单位净值",
        "累计净值",
        "_",
        "_",
        "_",
        "日增长率",
        "申购状态",
        "赎回状态",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df.sort_values(by=["净值日期"], inplace=True, ignore_index=True)
    big_df = big_df[
        ["净值日期", "单位净值", "累计净值", "日增长率", "申购状态", "赎回状态"]
    ]
    big_df["净值日期"] = pd.to_datetime(big_df["净值日期"], errors="coerce").dt.date
    big_df["单位净值"] = pd.to_numeric(big_df["单位净值"], errors="coerce")
    big_df["累计净值"] = pd.to_numeric(big_df["累计净值"], errors="coerce")
    big_df["日增长率"] = pd.to_numeric(big_df["日增长率"], errors="coerce")
    return big_df


def fund_etf_fund_daily_em() -> pd.DataFrame:
    """
    东方财富网-天天基金网-基金数据-场内交易基金
    https://fund.eastmoney.com/cnjy_dwjz.html
    :return: 当前交易日的所有场内交易基金数据
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/cnjy_dwjz.html"
    r = requests.get(url, headers=headers)
    r.encoding = "gb2312"
    show_day = pd.read_html(StringIO(r.text))[1].iloc[0, 6:10].tolist()
    temp_df = pd.read_html(StringIO(r.text))[1].iloc[1:, 2:]
    temp_df_columns = temp_df.iloc[0, :].tolist()[1:]
    temp_df = temp_df.iloc[1:, 1:]
    temp_df.columns = temp_df_columns
    temp_df["基金简称"] = temp_df["基金简称"].str.replace("行情吧档案", "")
    temp_df.reset_index(inplace=True, drop=True)
    temp_df.columns = [
        "基金代码",
        "基金简称",
        "类型",
        f"{show_day[0]}-单位净值",
        f"{show_day[0]}-累计净值",
        f"{show_day[2]}-单位净值",
        f"{show_day[2]}-累计净值",
        "增长值",
        "增长率",
        "市价",
        "折价率",
    ]
    return temp_df


def fund_etf_fund_info_em(
    fund: str = "511280",
    start_date: str = "20000101",
    end_date: str = "20500101",
) -> pd.DataFrame:
    """
    东方财富网站-天天基金网-基金数据-场内交易基金-历史净值明细
    https://fundf10.eastmoney.com/jjjz_511280.html
    :param fund: 场内交易基金代码, 可以通过 fund_etf_fund_daily_em 来获取
    :type fund: str
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 结束统计时间
    :type end_date: str
    :return: 东方财富网站-天天基金网-基金数据-场内交易基金-历史净值明细
    :rtype: pandas.DataFrame
    """
    url = "https://api.fund.eastmoney.com/f10/lsjz"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36",
        "Referer": f"https://fundf10.eastmoney.com/jjjz_{fund}.html",
    }
    params = {
        "fundCode": fund,
        "pageIndex": "1",
        "pageSize": "20",
        "startDate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "endDate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
        "_": round(time.time() * 1000),
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    total_page = math.ceil(data_json["TotalCount"] / 20)
    df_list = []
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageIndex": page})
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["Data"]["LSJZList"])
        df_list.append(temp_df)
    big_df = pd.concat(df_list)
    big_df.columns = [
        "净值日期",
        "单位净值",
        "累计净值",
        "_",
        "_",
        "_",
        "日增长率",
        "申购状态",
        "赎回状态",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        ["净值日期", "单位净值", "累计净值", "日增长率", "申购状态", "赎回状态"]
    ]
    big_df["净值日期"] = pd.to_datetime(big_df["净值日期"], errors="coerce").dt.date
    big_df["单位净值"] = pd.to_numeric(big_df["单位净值"], errors="coerce")
    big_df["累计净值"] = pd.to_numeric(big_df["累计净值"], errors="coerce")
    big_df["日增长率"] = pd.to_numeric(big_df["日增长率"], errors="coerce")
    big_df.sort_values(["净值日期"], inplace=True, ignore_index=True)
    return big_df


def fund_value_estimation_em(symbol: str = "全部") -> pd.DataFrame:
    """
    东方财富网-数据中心-净值估算
    https://fund.eastmoney.com/fundguzhi.html
    :param symbol: choice of {'全部', '股票型', '混合型', '债券型', '指数型', 'QDII', 'ETF联接', 'LOF', '场内交易基金'}
    :type symbol: str
    :return: 近期净值估算数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "全部": 1,
        "股票型": 2,
        "混合型": 3,
        "债券型": 4,
        "指数型": 5,
        "QDII": 6,
        "ETF联接": 7,
        "LOF": 8,
        "场内交易基金": 9,
    }
    url = "https://api.fund.eastmoney.com/FundGuZhi/GetFundGZList"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "https://fund.eastmoney.com/",
    }
    params = {
        "type": symbol_map[symbol],
        "sort": "3",
        "orderType": "desc",
        "canbuy": "0",
        "pageIndex": "1",
        "pageSize": "20000",
        "_": int(time.time() * 1000),
    }
    r = requests.get(url, params=params, headers=headers)
    json_data = r.json()
    temp_df = pd.DataFrame(json_data["Data"]["list"])
    value_day = json_data["Data"]["gzrq"]
    cal_day = json_data["Data"]["gxrq"]
    temp_df.columns = [
        "基金代码",
        "-",
        "-",
        "-",
        "-",
        "-",
        "基金类型",
        "-",
        "-",
        "-",
        "-",
        "估算日期",
        "-",
        "-",
        "-",
        "-",
        "_",
        "-",
        "-",
        "估算偏差",
        f"{cal_day}-估算数据-估算值",
        f"{cal_day}-估算数据-估算增长率",
        f"{cal_day}-公布数据-日增长率",
        f"{value_day}-单位净值",
        f"{cal_day}-公布数据-单位净值",
        "-",
        "基金名称",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "基金代码",
            "基金名称",
            f"{cal_day}-估算数据-估算值",
            f"{cal_day}-估算数据-估算增长率",
            f"{cal_day}-公布数据-单位净值",
            f"{cal_day}-公布数据-日增长率",
            "估算偏差",
            f"{value_day}-单位净值",
        ]
    ]
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.rename(columns={"index": "序号"}, inplace=True)
    return temp_df


def fund_hk_fund_hist_em(
    code: str = "1002200683", symbol: str = "历史净值明细"
) -> pd.DataFrame:
    """
    东方财富网-天天基金网-基金数据-香港基金-历史净值明细(分红送配详情)
    https://overseas.1234567.com.cn/f10/FundJz/968092#FHPS
    :param code: 通过 ak.fund_em_hk_rank() 获取
    :type code: str
    :param symbol: choice of {"历史净值明细", "分红送配详情"}
    :type symbol: str
    :return: 香港基金-历史净值明细(分红送配详情)
    :rtype: pandas.DataFrame
    """
    url = "https://overseas.1234567.com.cn/overseasapi/OpenApiHander.ashx"
    if symbol == "历史净值明细":
        params = {
            "api": "HKFDApi",
            "m": "MethodJZ",
            "hkfcode": f"{code}",
            "action": "2",
            "pageindex": "0",
            "pagesize": "1000",
            "date1": "",
            "date2": "",
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_one_df = pd.DataFrame(data_json["Data"])
        temp_one_df.columns = [
            "_",
            "_",
            "_",
            "净值日期",
            "单位净值",
            "_",
            "日增长值",
            "日增长率",
            "_",
            "单位",
            "_",
        ]
        temp_one_df = temp_one_df[
            [
                "净值日期",
                "单位净值",
                "日增长值",
                "日增长率",
                "单位",
            ]
        ]
    else:
        params = {
            "api": "HKFDApi",
            "m": "MethodJZ",
            "hkfcode": f"{code}",
            "action": "3",
            "pageindex": "0",
            "pagesize": "1000",
            "date1": "",
            "date2": "",
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_one_df = pd.DataFrame(data_json["Data"])
        temp_one_df.columns = [
            "_",
            "_",
            "_",
            "_",
            "_",
            "年份",
            "分红金额",
            "除息日",
            "权益登记日",
            "分红发放日",
            "_",
            "单位",
            "_",
            "_",
        ]
        temp_one_df = temp_one_df[
            [
                "年份",
                "权益登记日",
                "除息日",
                "分红发放日",
                "分红金额",
                "单位",
            ]
        ]
    return temp_one_df


if __name__ == "__main__":
    fund_purchase_em_df = fund_purchase_em()
    print(fund_purchase_em_df)

    fund_name_em_df = fund_name_em()
    print(fund_name_em_df)

    fund_info_index_em_df = fund_info_index_em(symbol="债券指数", indicator="全部")
    print(fund_info_index_em_df)

    fund_open_fund_daily_em_df = fund_open_fund_daily_em()
    print(fund_open_fund_daily_em_df)
    time.sleep(3)

    fund_open_fund_info_em_df = fund_open_fund_info_em(
        symbol="710001", indicator="单位净值走势"
    )
    print(fund_open_fund_info_em_df)
    time.sleep(3)

    fund_open_fund_info_em_df = fund_open_fund_info_em(
        symbol="502010", indicator="累计净值走势", period="成立来"
    )
    print(fund_open_fund_info_em_df)
    time.sleep(3)

    fund_open_fund_info_em_df = fund_open_fund_info_em(
        symbol="710001", indicator="累计收益率走势", period="成立来"
    )
    print(fund_open_fund_info_em_df)
    time.sleep(3)

    fund_open_fund_info_em_df = fund_open_fund_info_em(
        symbol="710001", indicator="同类排名走势"
    )
    print(fund_open_fund_info_em_df)
    time.sleep(3)

    fund_open_fund_info_em_df = fund_open_fund_info_em(
        symbol="710001", indicator="同类排名百分比"
    )
    print(fund_open_fund_info_em_df)
    time.sleep(3)

    fund_open_fund_info_em_df = fund_open_fund_info_em(
        symbol="014164", indicator="分红送配详情"
    )
    print(fund_open_fund_info_em_df)
    time.sleep(3)

    fund_open_fund_info_em_df = fund_open_fund_info_em(
        symbol="005561", indicator="拆分详情"
    )
    print(fund_open_fund_info_em_df)

    fund_money_fund_daily_em_df = fund_money_fund_daily_em()
    print(fund_money_fund_daily_em_df)

    fund_money_fund_info_em_df = fund_money_fund_info_em(symbol="000009")
    print(fund_money_fund_info_em_df)

    fund_financial_fund_daily_em_df = fund_financial_fund_daily_em()
    print(fund_financial_fund_daily_em_df)

    fund_financial_fund_info_em_df = fund_financial_fund_info_em(symbol="000134")
    print(fund_financial_fund_info_em_df)

    fund_graded_fund_daily_em_df = fund_graded_fund_daily_em()
    print(fund_graded_fund_daily_em_df)

    fund_graded_fund_info_em_df = fund_graded_fund_info_em(symbol="150232")
    print(fund_graded_fund_info_em_df)

    fund_etf_fund_daily_em_df = fund_etf_fund_daily_em()
    print(fund_etf_fund_daily_em_df)

    fund_etf_fund_info_em_df = fund_etf_fund_info_em(
        fund="511280", start_date="20000101", end_date="20500101"
    )
    print(fund_etf_fund_info_em_df)

    fund_value_estimation_em_df = fund_value_estimation_em(symbol="混合型")
    print(fund_value_estimation_em_df)

    fund_hk_fund_hist_em_df = fund_hk_fund_hist_em(
        code="1002200683", symbol="历史净值明细"
    )
    print(fund_hk_fund_hist_em_df)

    fund_hk_fund_hist_em_df = fund_hk_fund_hist_em(
        code="1002200683", symbol="分红送配详情"
    )
    print(fund_hk_fund_hist_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/20 18:00
Desc: 东方财富-数据中心-年报季报
东方财富-数据中心-年报季报-业绩快报-业绩报表
https://data.eastmoney.com/bbsj/202003/yjbb.html
"""

import pandas as pd
import requests

from akshare.utils.tqdm import get_tqdm


def stock_yjbb_em(date: str = "20200331") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-业绩快报-业绩报表
    https://data.eastmoney.com/bbsj/202003/yjbb.html
    :param date: "20200331", "20200630", "20200930", "20201231"; 从 20100331 开始
    :type date: str
    :return: 业绩报表
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.simplefilter(action="ignore", category=FutureWarning)  # 忽略所有
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "UPDATE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_LICO_FN_CPD",
        "columns": "ALL",
        "filter": f"(REPORTDATE='{'-'.join([date[:4], date[4:6], date[6:]])}')",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    big_list = []
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_list.append(temp_df)
    big_df = pd.concat(big_list, ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "_",
        "_",
        "_",
        "_",
        "最新公告日期",
        "_",
        "每股收益",
        "_",
        "营业总收入-营业总收入",
        "净利润-净利润",
        "净资产收益率",
        "营业总收入-同比增长",
        "净利润-同比增长",
        "每股净资产",
        "每股经营现金流量",
        "销售毛利率",
        "营业总收入-季度环比增长",
        "净利润-季度环比增长",
        "_",
        "_",
        "所处行业",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "每股收益",
            "营业总收入-营业总收入",
            "营业总收入-同比增长",
            "营业总收入-季度环比增长",
            "净利润-净利润",
            "净利润-同比增长",
            "净利润-季度环比增长",
            "每股净资产",
            "净资产收益率",
            "每股经营现金流量",
            "销售毛利率",
            "所处行业",
            "最新公告日期",
        ]
    ]
    big_df["每股收益"] = pd.to_numeric(big_df["每股收益"], errors="coerce")
    big_df["营业总收入-营业总收入"] = pd.to_numeric(
        big_df["营业总收入-营业总收入"], errors="coerce"
    )
    big_df["营业总收入-同比增长"] = pd.to_numeric(
        big_df["营业总收入-同比增长"], errors="coerce"
    )
    big_df["营业总收入-季度环比增长"] = pd.to_numeric(
        big_df["营业总收入-季度环比增长"], errors="coerce"
    )
    big_df["净利润-净利润"] = pd.to_numeric(big_df["净利润-净利润"], errors="coerce")
    big_df["净利润-同比增长"] = pd.to_numeric(
        big_df["净利润-同比增长"], errors="coerce"
    )
    big_df["净利润-季度环比增长"] = pd.to_numeric(
        big_df["净利润-季度环比增长"], errors="coerce"
    )
    big_df["每股净资产"] = pd.to_numeric(big_df["每股净资产"], errors="coerce")
    big_df["净资产收益率"] = pd.to_numeric(big_df["净资产收益率"], errors="coerce")
    big_df["每股经营现金流量"] = pd.to_numeric(
        big_df["每股经营现金流量"], errors="coerce"
    )
    big_df["销售毛利率"] = pd.to_numeric(big_df["销售毛利率"], errors="coerce")
    big_df["最新公告日期"] = pd.to_datetime(
        big_df["最新公告日期"], errors="coerce"
    ).dt.date
    return big_df


if __name__ == "__main__":
    stock_yjbb_em_df = stock_yjbb_em(date="20220331")
    print(stock_yjbb_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/24 18:10
Desc: 上海期货交易所指定交割仓库库存周报
https://datacenter.jin10.com/reportType/dc_shfe_weekly_stock
https://tsite.shfe.com.cn/statements/dataview.html?paramid=kx
"""

import pandas as pd
import requests


def futures_stock_shfe_js(date: str = "20240419") -> pd.DataFrame:
    """
    金十财经-上海期货交易所指定交割仓库库存周报
    https://datacenter.jin10.com/reportType/dc_shfe_weekly_stock
    :param date: 交易日; 库存周报只在每周的最后一个交易日公布数据
    :type date: str
    :return: 库存周报
    :rtype: pandas.Series
    """
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }
    url = "https://datacenter-api.jin10.com/reports/list"
    params = {
        "category": "stock",
        "date": "-".join([date[:4], date[4:6], date[6:]]),
        "attr_id": "1",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    columns_list = [item["name"] for item in data_json["data"]["keys"]]
    temp_df = pd.DataFrame(data_json["data"]["values"], columns=columns_list)
    for item in columns_list[1:]:
        temp_df[item] = pd.to_numeric(temp_df[item], errors="coerce")
    return temp_df


if __name__ == "__main__":
    futures_stock_shfe_js_df = futures_stock_shfe_js(date="20240419")
    print(futures_stock_shfe_js_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/7/4 15:00
Desc: 新浪财经-债券-沪深可转债-实时行情数据和历史行情数据
https://vip.stock.finance.sina.com.cn/mkt/#hskzz_z
"""

import datetime
import re

import pandas as pd
import py_mini_racer
import requests

from akshare.bond.cons import (
    zh_sina_bond_hs_cov_count_url,
    zh_sina_bond_hs_cov_payload,
    zh_sina_bond_hs_cov_url,
    zh_sina_bond_hs_cov_hist_url,
)
from akshare.stock.cons import hk_js_decode
from akshare.utils import demjson
from akshare.utils.func import fetch_paginated_data
from akshare.utils.tqdm import get_tqdm


def _get_zh_bond_hs_cov_page_count() -> int:
    """
    新浪财经-行情中心-债券-沪深可转债的总页数
    https://vip.stock.finance.sina.com.cn/mkt/#hskzz_z
    :return: 总页数
    :rtype: int
    """
    params = {
        "node": "hskzz_z",
    }
    r = requests.get(zh_sina_bond_hs_cov_count_url, params=params)
    page_count = int(re.findall(re.compile(r"\d+"), r.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def bond_zh_hs_cov_spot() -> pd.DataFrame:
    """
    新浪财经-债券-沪深可转债的实时行情数据; 大量抓取容易封IP
    https://vip.stock.finance.sina.com.cn/mkt/#hskzz_z
    :return: 所有沪深可转债在当前时刻的实时行情数据
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = _get_zh_bond_hs_cov_page_count()
    zh_sina_bond_hs_payload_copy = zh_sina_bond_hs_cov_payload.copy()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_count + 1), leave=False):
        zh_sina_bond_hs_payload_copy.update({"page": page})
        res = requests.get(zh_sina_bond_hs_cov_url, params=zh_sina_bond_hs_payload_copy)
        data_json = demjson.decode(res.text)
        big_df = pd.concat(objs=[big_df, pd.DataFrame(data_json)], ignore_index=True)
    return big_df


def bond_zh_hs_cov_daily(symbol: str = "sh010107") -> pd.DataFrame:
    """
    新浪财经-债券-沪深可转债的历史行情数据, 大量抓取容易封 IP
    https://vip.stock.finance.sina.com.cn/mkt/#hskzz_z
    :param symbol: 沪深可转债代码; e.g., sh010107
    :type symbol: str
    :return: 指定沪深可转债代码的日 K 线数据
    :rtype: pandas.DataFrame
    """
    r = requests.get(
        zh_sina_bond_hs_cov_hist_url.format(
            symbol, datetime.datetime.now().strftime("%Y_%m_%d")
        )
    )
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", r.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行js解密代码
    data_df = pd.DataFrame(dict_list)
    data_df["date"] = pd.to_datetime(data_df["date"]).dt.date
    return data_df


def _code_id_map() -> dict:
    """
    东方财富-股票和市场代码
    https://quote.eastmoney.com/center/gridlist.html#hs_a_board
    :return: 股票和市场代码
    :rtype: dict
    """
    url = "https://80.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f12",
        "fs": "m:1 t:2,m:1 t:23",
        "fields": "f3,f12",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df["market_id"] = 1
    temp_df.rename(columns={"f12": "sh_code", "market_id": "sh_id"}, inplace=True)
    code_id_dict = dict(zip(temp_df["sh_code"], temp_df["sh_id"]))
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 t:6,m:0 t:80",
        "fields": "f3,f12",
    }
    temp_df_sz = fetch_paginated_data(url, params)
    temp_df_sz["sz_id"] = 0
    code_id_dict.update(dict(zip(temp_df_sz["f12"], temp_df_sz["sz_id"])))
    return code_id_dict


def bond_zh_hs_cov_min(
    symbol: str = "sz128039",
    period: str = "15",
    adjust: str = "",
    start_date: str = "1979-09-01 09:32:00",
    end_date: str = "2222-01-01 09:32:00",
) -> pd.DataFrame:
    """
    东方财富网-可转债-分时行情
    https://quote.eastmoney.com/concept/sz128039.html
    :param symbol: 转债代码
    :type symbol: str
    :param period: choice of {'1', '5', '15', '30', '60'}
    :type period: str
    :param adjust: choice of {'', 'qfq', 'hfq'}
    :type adjust: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 分时行情
    :rtype: pandas.DataFrame
    """
    market_type = {"sh": "1", "sz": "0"}
    if period == "1":
        url = "https://push2.eastmoney.com/api/qt/stock/trends2/get"
        params = {
            "secid": f"{market_type[symbol[:2]]}.{symbol[2:]}",
            "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
            "iscr": "0",
            "iscca": "0",
            "ut": "f057cbcbce2a86e2866ab8877db1d059",
            "ndays": "1",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["trends"]]
        )
        temp_df.columns = [
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "最新价",
        ]
        temp_df.index = pd.to_datetime(temp_df["时间"])
        temp_df = temp_df[start_date:end_date]
        temp_df.reset_index(drop=True, inplace=True)
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
        temp_df["时间"] = pd.to_datetime(temp_df["时间"]).astype(
            str
        )  # show datatime here
        return temp_df
    else:
        adjust_map = {
            "": "0",
            "qfq": "1",
            "hfq": "2",
        }
        url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
        params = {
            "secid": f"{market_type[symbol[:2]]}.{symbol[2:]}",
            "klt": period,
            "fqt": adjust_map[adjust],
            "lmt": "66",
            "end": "20500000",
            "iscca": "1",
            "fields1": "f1,f2,f3,f4,f5",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
            "ut": "7eea3edcaed734bea9cbfc24409ed989",
            "forcect": "1",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["klines"]]
        )
        temp_df.columns = [
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "振幅",
            "涨跌幅",
            "涨跌额",
            "换手率",
        ]
        temp_df.index = pd.to_datetime(temp_df["时间"])
        temp_df = temp_df[start_date:end_date]
        temp_df.reset_index(drop=True, inplace=True)
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
        temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
        temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
        temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
        temp_df["时间"] = pd.to_datetime(temp_df["时间"]).astype(str)
        temp_df = temp_df[
            [
                "时间",
                "开盘",
                "收盘",
                "最高",
                "最低",
                "涨跌幅",
                "涨跌额",
                "成交量",
                "成交额",
                "振幅",
                "换手率",
            ]
        ]
        return temp_df


def bond_zh_hs_cov_pre_min(symbol: str = "sh113570") -> pd.DataFrame:
    """
    东方财富网-可转债-分时行情-盘前
    https://quote.eastmoney.com/concept/sz128039.html
    :param symbol: 转债代码
    :type symbol: str
    :return: 分时行情-盘前
    :rtype: pandas.DataFrame
    """
    market_type = {"sh": "1", "sz": "0"}
    url = "https://push2.eastmoney.com/api/qt/stock/trends2/get"
    params = {
        "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
        "ndays": "1",
        "iscr": "1",
        "iscca": "0",
        "secid": f"{market_type[symbol[:2]]}.{symbol[2:]}",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["trends"]])
    temp_df.columns = [
        "时间",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "最新价",
    ]
    temp_df.index = pd.to_datetime(temp_df["时间"])
    temp_df.reset_index(drop=True, inplace=True)
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["时间"] = pd.to_datetime(temp_df["时间"]).astype(str)
    return temp_df


def bond_zh_cov() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-可转债数据
    https://data.eastmoney.com/kzz/default.html
    :return: 可转债数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "PUBLIC_START_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_BOND_CB_LIST",
        "columns": "ALL",
        "quoteColumns": "f2~01~CONVERT_STOCK_CODE~CONVERT_STOCK_PRICE,"
        "f235~10~SECURITY_CODE~TRANSFER_PRICE,f236~10~SECURITY_CODE~TRANSFER_VALUE,"
        "f2~10~SECURITY_CODE~CURRENT_BOND_PRICE,f237~10~SECURITY_CODE~TRANSFER_PREMIUM_RATIO,"
        "f239~10~SECURITY_CODE~RESALE_TRIG_PRICE,f240~10~SECURITY_CODE~REDEEM_TRIG_PRICE,"
        "f23~01~CONVERT_STOCK_CODE~PBV_RATIO",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.columns = [
        "债券代码",
        "_",
        "_",
        "债券简称",
        "_",
        "上市时间",
        "正股代码",
        "_",
        "信用评级",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "发行规模",
        "申购上限",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "申购代码",
        "_",
        "申购日期",
        "_",
        "_",
        "中签号发布日",
        "原股东配售-股权登记日",
        "正股简称",
        "原股东配售-每股配售额",
        "_",
        "中签率",
        "-",
        "_",
        "_",
        "_",
        "_",
        "_",
        "正股价",
        "转股价",
        "转股价值",
        "债现价",
        "转股溢价率",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "债券代码",
            "债券简称",
            "申购日期",
            "申购代码",
            "申购上限",
            "正股代码",
            "正股简称",
            "正股价",
            "转股价",
            "转股价值",
            "债现价",
            "转股溢价率",
            "原股东配售-股权登记日",
            "原股东配售-每股配售额",
            "发行规模",
            "中签号发布日",
            "中签率",
            "上市时间",
            "信用评级",
        ]
    ]

    big_df["申购上限"] = pd.to_numeric(big_df["申购上限"], errors="coerce")
    big_df["正股价"] = pd.to_numeric(big_df["正股价"], errors="coerce")
    big_df["转股价"] = pd.to_numeric(big_df["转股价"], errors="coerce")
    big_df["转股价值"] = pd.to_numeric(big_df["转股价值"], errors="coerce")
    big_df["债现价"] = pd.to_numeric(big_df["债现价"], errors="coerce")
    big_df["转股溢价率"] = pd.to_numeric(big_df["转股溢价率"], errors="coerce")
    big_df["原股东配售-每股配售额"] = pd.to_numeric(
        big_df["原股东配售-每股配售额"], errors="coerce"
    )
    big_df["发行规模"] = pd.to_numeric(big_df["发行规模"], errors="coerce")
    big_df["中签率"] = pd.to_numeric(big_df["中签率"], errors="coerce")
    big_df["中签号发布日"] = pd.to_datetime(
        big_df["中签号发布日"], errors="coerce"
    ).dt.date
    big_df["上市时间"] = pd.to_datetime(big_df["上市时间"], errors="coerce").dt.date
    big_df["申购日期"] = pd.to_datetime(big_df["申购日期"], errors="coerce").dt.date
    big_df["原股东配售-股权登记日"] = pd.to_datetime(
        big_df["原股东配售-股权登记日"], errors="coerce"
    ).dt.date
    big_df["债现价"] = big_df["债现价"].fillna(100)
    return big_df


def bond_cov_comparison() -> pd.DataFrame:
    """
    东方财富网-行情中心-债券市场-可转债比价表
    https://quote.eastmoney.com/center/fullscreenlist.html#convertible_comparison
    :return: 可转债比价表数据
    :rtype: pandas.DataFrame
    """
    url = "https://16.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f243",
        "fs": "b:MK0354",
        "fields": "f1,f152,f2,f3,f12,f13,f14,f227,f228,f229,f230,f231,f232,f233,f234,"
        "f235,f236,f237,f238,f239,f240,f241,f242,f26,f243",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "序号",
        "_",
        "转债最新价",
        "转债涨跌幅",
        "转债代码",
        "_",
        "转债名称",
        "上市日期",
        "_",
        "纯债价值",
        "_",
        "正股最新价",
        "正股涨跌幅",
        "_",
        "正股代码",
        "_",
        "正股名称",
        "转股价",
        "转股价值",
        "转股溢价率",
        "纯债溢价率",
        "回售触发价",
        "强赎触发价",
        "到期赎回价",
        "开始转股日",
        "申购日期",
    ]
    temp_df = temp_df[
        [
            "序号",
            "转债代码",
            "转债名称",
            "转债最新价",
            "转债涨跌幅",
            "正股代码",
            "正股名称",
            "正股最新价",
            "正股涨跌幅",
            "转股价",
            "转股价值",
            "转股溢价率",
            "纯债溢价率",
            "回售触发价",
            "强赎触发价",
            "到期赎回价",
            "纯债价值",
            "开始转股日",
            "上市日期",
            "申购日期",
        ]
    ]
    return temp_df


def bond_zh_cov_info(
    symbol: str = "123121", indicator: str = "基本信息"
) -> pd.DataFrame:
    """
    https://data.eastmoney.com/kzz/detail/123121.html
    东方财富网-数据中心-新股数据-可转债详情
    :param symbol: 可转债代码
    :type symbol: str
    :param indicator: choice of {"基本信息", "中签号", "筹资用途", "重要日期"}
    :type indicator: str
    :return: 可转债详情
    :rtype: pandas.DataFrame
    """
    indicator_map = {
        "基本信息": "RPT_BOND_CB_LIST",
        "中签号": "RPT_CB_BALLOTNUM",
        "筹资用途": "RPT_BOND_BS_OPRFINVESTITEM",
        "重要日期": "RPT_CB_IMPORTANTDATE",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_BOND_CB_LIST",
        "columns": "ALL",
        "quoteColumns": "f2~01~CONVERT_STOCK_CODE~CONVERT_STOCK_PRICE,f235~10~SECURITY_CODE~TRANSFER_PRICE,"
        "f236~10~SECURITY_CODE~TRANSFER_VALUE,f2~10~SECURITY_CODE~CURRENT_BOND_PRICE,"
        "f237~10~SECURITY_CODE~TRANSFER_PREMIUM_RATIO,f239~10~SECURITY_CODE~RESALE_TRIG_PRICE,"
        "f240~10~SECURITY_CODE~REDEEM_TRIG_PRICE,f23~01~CONVERT_STOCK_CODE~PBV_RATIO",
        "quoteType": "0",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(SECURITY_CODE="{symbol}")',
    }
    if indicator == "基本信息":
        params.update(
            {
                "reportName": indicator_map[indicator],
                "quoteColumns": "f2~01~CONVERT_STOCK_CODE~CONVERT_STOCK_PRICE,f235~10~SECURITY_CODE~TRANSFER_PRICE,"
                "f236~10~SECURITY_CODE~TRANSFER_VALUE,f2~10~SECURITY_CODE~CURRENT_BOND_PRICE,"
                "f237~10~SECURITY_CODE~TRANSFER_PREMIUM_RATIO,f239~10~SECURITY_CODE~RESALE_TRIG_PRICE,"
                "f240~10~SECURITY_CODE~REDEEM_TRIG_PRICE,f23~01~CONVERT_STOCK_CODE~PBV_RATIO",
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame.from_dict(data_json["result"]["data"])
        return temp_df
    elif indicator == "中签号":
        params.update(
            {
                "reportName": indicator_map[indicator],
                "quoteColumns": "",
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame.from_dict(data_json["result"]["data"])
        return temp_df
    elif indicator == "筹资用途":
        params.update(
            {
                "reportName": indicator_map[indicator],
                "quoteColumns": "",
                "sortColumns": "SORT",
                "sortTypes": "1",
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame.from_dict(data_json["result"]["data"])
        return temp_df
    elif indicator == "重要日期":
        params.update(
            {
                "reportName": indicator_map[indicator],
                "quoteColumns": "",
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame.from_dict(data_json["result"]["data"])
        return temp_df
    else:
        return pd.DataFrame()


def bond_zh_cov_value_analysis(symbol: str = "113527") -> pd.DataFrame:
    """
    https://data.eastmoney.com/kzz/detail/113527.html
    东方财富网-数据中心-新股数据-可转债数据-价值分析-溢价率分析
    :param symbol: 可转债代码
    :type symbol: str
    :return: 可转债价值分析
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/get"
    params = {
        "sty": "ALL",
        "token": "894050c76af8597a853f5b408b759f5d",
        "st": "date",
        "sr": "1",
        "source": "WEB",
        "type": "RPTA_WEB_KZZ_LS",
        "filter": f'(zcode="{symbol}")',
        "p": "1",
        "ps": "8000",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "日期",
        "-",
        "-",
        "转股价值",
        "纯债价值",
        "纯债溢价率",
        "转股溢价率",
        "收盘价",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "日期",
            "收盘价",
            "纯债价值",
            "转股价值",
            "纯债溢价率",
            "转股溢价率",
        ]
    ]
    temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"], errors="coerce")
    temp_df["纯债价值"] = pd.to_numeric(temp_df["纯债价值"], errors="coerce")
    temp_df["转股价值"] = pd.to_numeric(temp_df["转股价值"], errors="coerce")
    temp_df["纯债溢价率"] = pd.to_numeric(temp_df["纯债溢价率"], errors="coerce")
    temp_df["转股溢价率"] = pd.to_numeric(temp_df["转股溢价率"], errors="coerce")
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    return temp_df


if __name__ == "__main__":
    bond_zh_hs_cov_min_df = bond_zh_hs_cov_min(
        symbol="sz128039",
        period="1",
        adjust="hfq",
        start_date="1979-09-01 09:32:00",
        end_date="2222-01-01 09:32:00",
    )
    print(bond_zh_hs_cov_min_df)

    bond_zh_hs_cov_pre_min_df = bond_zh_hs_cov_pre_min(symbol="sz128039")
    print(bond_zh_hs_cov_pre_min_df)

    bond_zh_hs_cov_daily_df = bond_zh_hs_cov_daily(symbol="sz128039")
    print(bond_zh_hs_cov_daily_df)

    bond_zh_hs_cov_spot_df = bond_zh_hs_cov_spot()
    print(bond_zh_hs_cov_spot_df)

    bond_zh_cov_df = bond_zh_cov()
    print(bond_zh_cov_df)

    bond_cov_comparison_df = bond_cov_comparison()
    print(bond_cov_comparison_df)

    bond_zh_cov_info_df = bond_zh_cov_info(symbol="123121", indicator="基本信息")
    print(bond_zh_cov_info_df)

    bond_zh_cov_value_analysis_df = bond_zh_cov_value_analysis(symbol="113527")
    print(bond_zh_cov_value_analysis_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/9/13 16:00
Desc: 深圳证券交易所-期权子网-行情数据-当日合约
"""

from io import BytesIO

import pandas as pd
import requests


def option_current_day_szse() -> pd.DataFrame:
    """
    深圳证券交易所-期权子网-行情数据-当日合约
    https://www.sse.org.cn/option/quotation/contract/daycontract/index.html
    :return: 深圳期权当日合约
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.filterwarnings(
        action="ignore", message="Workbook contains no default style"
    )
    url = "https://www.sse.org.cn/api/report/ShowReport"
    params = {
        "SHOWTYPE": "xlsx",
        "CATALOGID": "option_drhy",
        "TABKEY": "tab1",
    }
    r = requests.get(url, params=params)
    temp_df = pd.read_excel(BytesIO(r.content))
    temp_df["序号"] = pd.to_numeric(temp_df["序号"], errors="coerce")
    temp_df["行权价"] = pd.to_numeric(temp_df["行权价"], errors="coerce")
    temp_df["合约单位"] = pd.to_numeric(temp_df["合约单位"], errors="coerce")
    temp_df["涨停价格"] = pd.to_numeric(temp_df["涨停价格"], errors="coerce")
    temp_df["跌停价格"] = pd.to_numeric(temp_df["跌停价格"], errors="coerce")
    temp_df["前结算价"] = pd.to_numeric(temp_df["前结算价"], errors="coerce")
    temp_df["合约总持仓"] = pd.to_numeric(temp_df["合约总持仓"], errors="coerce")
    temp_df["原行权价格"] = pd.to_numeric(temp_df["原行权价格"], errors="coerce")
    temp_df["原合约单位"] = pd.to_numeric(temp_df["原合约单位"], errors="coerce")
    temp_df["合约到期剩余交易天数"] = pd.to_numeric(
        temp_df["合约到期剩余交易天数"], errors="coerce"
    )
    temp_df["合约到期剩余自然天数"] = pd.to_numeric(
        temp_df["合约到期剩余自然天数"], errors="coerce"
    )
    temp_df["下次合约调整剩余交易天数"] = pd.to_numeric(
        temp_df["下次合约调整剩余交易天数"], errors="coerce"
    )
    temp_df["下次合约调整剩余自然天数"] = pd.to_numeric(
        temp_df["下次合约调整剩余自然天数"], errors="coerce"
    )
    temp_df["交易日期"] = pd.to_datetime(temp_df["交易日期"], errors="coerce").dt.date
    temp_df["最后交易日"] = pd.to_datetime(
        temp_df["最后交易日"], errors="coerce"
    ).dt.date
    temp_df["行权日"] = pd.to_datetime(temp_df["行权日"], errors="coerce").dt.date
    temp_df["到期日"] = pd.to_datetime(temp_df["到期日"], errors="coerce").dt.date
    temp_df["交收日"] = pd.to_datetime(temp_df["交收日"], errors="coerce").dt.date
    temp_df = temp_df[
        [
            "序号",
            "合约编码",
            "合约代码",
            "合约简称",
            "标的证券简称(代码)",
            "合约类型",
            "行权价",
            "合约单位",
            "最后交易日",
            "行权日",
            "到期日",
            "交收日",
            "新挂",
            "涨停价格",
            "跌停价格",
            "前结算价",
            "合约调整",
            "停牌",
            "合约总持仓",
            "挂牌原因",
            "原合约代码",
            "原合约简称",
            "原行权价格",
            "原合约单位",
            "合约到期剩余交易天数",
            "合约到期剩余自然天数",
            "下次合约调整剩余交易天数",
            "下次合约调整剩余自然天数",
            "交易日期",
        ]
    ]
    return temp_df


if __name__ == "__main__":
    option_current_day_szse_df = option_current_day_szse()
    print(option_current_day_szse_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/1/13 22:30
Desc: 东方财富网-数据中心-大宗交易-市场统计
https://data.eastmoney.com/dzjy/
"""

import pandas as pd
import requests


def stock_dzjy_sctj() -> pd.DataFrame:
    """
    东方财富网-数据中心-大宗交易-市场统计
    https://data.eastmoney.com/dzjy/dzjy_sctj.html
    :return: 市场统计表
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "PRT_BLOCKTRADE_MARKET_STA",
        "columns": "TRADE_DATE,SZ_INDEX,SZ_CHANGE_RATE,BLOCKTRADE_DEAL_AMT,PREMIUM_DEAL_AMT,"
        "PREMIUM_RATIO,DISCOUNT_DEAL_AMT,DISCOUNT_RATIO",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = int(data_json["result"]["pages"])
    big_df = pd.DataFrame()
    for page in range(1, total_page + 1):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df["index"] + 1
    big_df.columns = [
        "序号",
        "交易日期",
        "上证指数",
        "上证指数涨跌幅",
        "大宗交易成交总额",
        "溢价成交总额",
        "溢价成交总额占比",
        "折价成交总额",
        "折价成交总额占比",
    ]
    big_df["交易日期"] = pd.to_datetime(big_df["交易日期"], errors="coerce").dt.date
    big_df["上证指数"] = pd.to_numeric(big_df["上证指数"], errors="coerce")
    big_df["上证指数涨跌幅"] = pd.to_numeric(big_df["上证指数涨跌幅"], errors="coerce")
    big_df["大宗交易成交总额"] = pd.to_numeric(
        big_df["大宗交易成交总额"], errors="coerce"
    )
    big_df["溢价成交总额"] = pd.to_numeric(big_df["溢价成交总额"], errors="coerce")
    big_df["溢价成交总额占比"] = pd.to_numeric(
        big_df["溢价成交总额占比"], errors="coerce"
    )
    big_df["折价成交总额"] = pd.to_numeric(big_df["折价成交总额"], errors="coerce")
    big_df["折价成交总额占比"] = pd.to_numeric(
        big_df["折价成交总额占比"], errors="coerce"
    )
    return big_df


def stock_dzjy_mrmx(
    symbol: str = "基金", start_date: str = "20220104", end_date: str = "20220104"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-大宗交易-每日明细
    https://data.eastmoney.com/dzjy/dzjy_mrmx.html
    :param symbol: choice of {'A股', 'B股', '基金', '债券'}
    :type symbol: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 每日明细
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "A股": "1",
        "B股": "2",
        "基金": "3",
        "债券": "4",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "SECURITY_CODE",
        "sortTypes": "1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_DATA_BLOCKTRADE",
        "columns": "TRADE_DATE,SECURITY_CODE,SECUCODE,SECURITY_NAME_ABBR,CHANGE_RATE,CLOSE_PRICE,"
        "DEAL_PRICE,PREMIUM_RATIO,DEAL_VOLUME,DEAL_AMT,TURNOVER_RATE,BUYER_NAME,SELLER_NAME,"
        "CHANGE_RATE_1DAYS,CHANGE_RATE_5DAYS,CHANGE_RATE_10DAYS,CHANGE_RATE_20DAYS,BUYER_CODE,SELLER_CODE",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(SECURITY_TYPE_WEB={symbol_map[symbol]})(TRADE_DATE>=
        '{"-".join([start_date[:4], start_date[4:6], start_date[6:]])}')(TRADE_DATE<=
        '{"-".join([end_date[:4], end_date[4:6], end_date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["result"]["data"]:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    if symbol in {"A股"}:
        temp_df.columns = [
            "序号",
            "交易日期",
            "证券代码",
            "-",
            "证券简称",
            "涨跌幅",
            "收盘价",
            "成交价",
            "折溢率",
            "成交量",
            "成交额",
            "成交额/流通市值",
            "买方营业部",
            "卖方营业部",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
        ]
        temp_df["交易日期"] = pd.to_datetime(
            temp_df["交易日期"], errors="coerce"
        ).dt.date
        temp_df = temp_df[
            [
                "序号",
                "交易日期",
                "证券代码",
                "证券简称",
                "涨跌幅",
                "收盘价",
                "成交价",
                "折溢率",
                "成交量",
                "成交额",
                "成交额/流通市值",
                "买方营业部",
                "卖方营业部",
            ]
        ]
        temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
        temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"], errors="coerce")
        temp_df["成交价"] = pd.to_numeric(temp_df["成交价"], errors="coerce")
        temp_df["折溢率"] = pd.to_numeric(temp_df["折溢率"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["成交额/流通市值"] = pd.to_numeric(
            temp_df["成交额/流通市值"], errors="coerce"
        )
    if symbol in {"B股", "基金", "债券"}:
        temp_df.columns = [
            "序号",
            "交易日期",
            "证券代码",
            "-",
            "证券简称",
            "-",
            "-",
            "成交价",
            "-",
            "成交量",
            "成交额",
            "-",
            "买方营业部",
            "卖方营业部",
            "_",
            "_",
            "_",
            "_",
            "_",
            "_",
        ]
        temp_df["交易日期"] = pd.to_datetime(
            temp_df["交易日期"], errors="coerce"
        ).dt.date
        temp_df = temp_df[
            [
                "序号",
                "交易日期",
                "证券代码",
                "证券简称",
                "成交价",
                "成交量",
                "成交额",
                "买方营业部",
                "卖方营业部",
            ]
        ]
        temp_df["成交价"] = pd.to_numeric(temp_df["成交价"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    return temp_df


def stock_dzjy_mrtj(
    start_date: str = "20220105", end_date: str = "20220105"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-大宗交易-每日统计
    https://data.eastmoney.com/dzjy/dzjy_mrtj.html
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 每日统计
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TURNOVERRATE",
        "sortTypes": "-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_BLOCKTRADE_STA",
        "columns": "TRADE_DATE,SECURITY_CODE,SECUCODE,SECURITY_NAME_ABBR,CHANGE_RATE,"
        "CLOSE_PRICE,AVERAGE_PRICE,PREMIUM_RATIO,DEAL_NUM,VOLUME,DEAL_AMT,"
        "TURNOVERRATE,D1_CLOSE_ADJCHRATE,D5_CLOSE_ADJCHRATE,D10_CLOSE_ADJCHRATE,D20_CLOSE_ADJCHRATE",
        "source": "WEB",
        "client": "WEB",
        "filter": f"(TRADE_DATE>='{'-'.join([start_date[:4], start_date[4:6], start_date[6:]])}')(TRADE_DATE<="
        f"'{'-'.join([end_date[:4], end_date[4:6], end_date[6:]])}')",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "交易日期",
        "证券代码",
        "-",
        "证券简称",
        "涨跌幅",
        "收盘价",
        "成交价",
        "折溢率",
        "成交笔数",
        "成交总量",
        "成交总额",
        "成交总额/流通市值",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df["交易日期"] = pd.to_datetime(temp_df["交易日期"], errors="coerce").dt.date
    temp_df = temp_df[
        [
            "序号",
            "交易日期",
            "证券代码",
            "证券简称",
            "涨跌幅",
            "收盘价",
            "成交价",
            "折溢率",
            "成交笔数",
            "成交总量",
            "成交总额",
            "成交总额/流通市值",
        ]
    ]
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"], errors="coerce")
    temp_df["成交价"] = pd.to_numeric(temp_df["成交价"], errors="coerce")
    temp_df["折溢率"] = pd.to_numeric(temp_df["折溢率"], errors="coerce")
    temp_df["成交笔数"] = pd.to_numeric(temp_df["成交笔数"], errors="coerce")
    temp_df["成交总量"] = pd.to_numeric(temp_df["成交总量"], errors="coerce")
    temp_df["成交总额"] = pd.to_numeric(temp_df["成交总额"], errors="coerce")
    temp_df["成交总额/流通市值"] = pd.to_numeric(
        temp_df["成交总额/流通市值"], errors="coerce"
    )
    return temp_df


def stock_dzjy_hygtj(symbol: str = "近三月") -> pd.DataFrame:
    """
    东方财富网-数据中心-大宗交易-活跃 A 股统计
    https://data.eastmoney.com/dzjy/dzjy_hygtj.html
    :param symbol: choice of {'近一月', '近三月', '近六月', '近一年'}
    :type symbol: str
    :return: 活跃 A 股统计
    :rtype: pandas.DataFrame
    """
    period_map = {
        "近一月": "1",
        "近三月": "3",
        "近六月": "6",
        "近一年": "12",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "DEAL_NUM,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_BLOCKTRADE_ACSTA",
        "columns": "SECURITY_CODE,SECUCODE,SECURITY_NAME_ABBR,CLOSE_PRICE,CHANGE_RATE,TRADE_DATE,"
        "DEAL_AMT,PREMIUM_RATIO,SUM_TURNOVERRATE,DEAL_NUM,PREMIUM_TIMES,DISCOUNT_TIMES,"
        "D1_AVG_ADJCHRATE,D5_AVG_ADJCHRATE,D10_AVG_ADJCHRATE,D20_AVG_ADJCHRATE,DATE_TYPE_CODE",
        "source": "WEB",
        "client": "WEB",
        "filter": f"(DATE_TYPE_CODE={period_map[symbol]})",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in range(1, int(total_page) + 1):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "证券代码",
        "_",
        "证券简称",
        "最新价",
        "涨跌幅",
        "最近上榜日",
        "总成交额",
        "折溢率",
        "成交总额/流通市值",
        "上榜次数-总计",
        "上榜次数-溢价",
        "上榜次数-折价",
        "上榜日后平均涨跌幅-1日",
        "上榜日后平均涨跌幅-5日",
        "上榜日后平均涨跌幅-10日",
        "上榜日后平均涨跌幅-20日",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "证券代码",
            "证券简称",
            "最新价",
            "涨跌幅",
            "最近上榜日",
            "上榜次数-总计",
            "上榜次数-溢价",
            "上榜次数-折价",
            "总成交额",
            "折溢率",
            "成交总额/流通市值",
            "上榜日后平均涨跌幅-1日",
            "上榜日后平均涨跌幅-5日",
            "上榜日后平均涨跌幅-10日",
            "上榜日后平均涨跌幅-20日",
        ]
    ]
    big_df["最近上榜日"] = pd.to_datetime(big_df["最近上榜日"], errors="coerce").dt.date
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["上榜次数-总计"] = pd.to_numeric(big_df["上榜次数-总计"], errors="coerce")
    big_df["上榜次数-溢价"] = pd.to_numeric(big_df["上榜次数-溢价"], errors="coerce")
    big_df["上榜次数-折价"] = pd.to_numeric(big_df["上榜次数-折价"], errors="coerce")
    big_df["总成交额"] = pd.to_numeric(big_df["总成交额"], errors="coerce")
    big_df["折溢率"] = pd.to_numeric(big_df["折溢率"], errors="coerce")
    big_df["成交总额/流通市值"] = pd.to_numeric(
        big_df["成交总额/流通市值"], errors="coerce"
    )
    big_df["上榜日后平均涨跌幅-1日"] = pd.to_numeric(
        big_df["上榜日后平均涨跌幅-1日"], errors="coerce"
    )
    big_df["上榜日后平均涨跌幅-5日"] = pd.to_numeric(
        big_df["上榜日后平均涨跌幅-5日"], errors="coerce"
    )
    big_df["上榜日后平均涨跌幅-10日"] = pd.to_numeric(
        big_df["上榜日后平均涨跌幅-10日"], errors="coerce"
    )
    big_df["上榜日后平均涨跌幅-20日"] = pd.to_numeric(
        big_df["上榜日后平均涨跌幅-20日"], errors="coerce"
    )
    return big_df


def stock_dzjy_hyyybtj(symbol: str = "近3日") -> pd.DataFrame:
    """
    东方财富网-数据中心-大宗交易-活跃营业部统计
    https://data.eastmoney.com/dzjy/dzjy_hyyybtj.html
    :param symbol: choice of {'当前交易日', '近3日', '近5日', '近10日', '近30日'}
    :type symbol: str
    :return: 活跃营业部统计
    :rtype: pandas.DataFrame
    """
    period_map = {
        "当前交易日": "1",
        "近3日": "3",
        "近5日": "5",
        "近10日": "10",
        "近30日": "30",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "BUYER_NUM,TOTAL_BUYAMT",
        "sortTypes": "-1,-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_BLOCKTRADE_OPERATEDEPTSTATISTICS",
        "columns": "OPERATEDEPT_CODE,OPERATEDEPT_NAME,ONLIST_DATE,STOCK_DETAILS,"
        "BUYER_NUM,SELLER_NUM,TOTAL_BUYAMT,TOTAL_SELLAMT,TOTAL_NETAMT,N_DATE",
        "source": "WEB",
        "client": "WEB",
        "filter": f"(N_DATE=-{period_map[symbol]})",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in range(1, int(total_page) + 1):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "_",
        "营业部名称",
        "最近上榜日",
        "买入的股票",
        "次数总计-买入",
        "次数总计-卖出",
        "成交金额统计-买入",
        "成交金额统计-卖出",
        "成交金额统计-净买入额",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "最近上榜日",
            "营业部名称",
            "次数总计-买入",
            "次数总计-卖出",
            "成交金额统计-买入",
            "成交金额统计-卖出",
            "成交金额统计-净买入额",
            "买入的股票",
        ]
    ]
    big_df["最近上榜日"] = pd.to_datetime(big_df["最近上榜日"], errors="coerce").dt.date
    big_df["次数总计-买入"] = pd.to_numeric(big_df["次数总计-买入"], errors="coerce")
    big_df["次数总计-卖出"] = pd.to_numeric(big_df["次数总计-卖出"], errors="coerce")
    big_df["成交金额统计-买入"] = pd.to_numeric(
        big_df["成交金额统计-买入"], errors="coerce"
    )
    big_df["成交金额统计-卖出"] = pd.to_numeric(
        big_df["成交金额统计-卖出"], errors="coerce"
    )
    big_df["成交金额统计-净买入额"] = pd.to_numeric(
        big_df["成交金额统计-净买入额"], errors="coerce"
    )
    return big_df


def stock_dzjy_yybph(symbol: str = "近三月") -> pd.DataFrame:
    """
    东方财富网-数据中心-大宗交易-营业部排行
    https://data.eastmoney.com/dzjy/dzjy_yybph.html
    :param symbol: choice of {'近一月', '近三月', '近六月', '近一年'}
    :type symbol: str
    :return: 营业部排行
    :rtype: pandas.DataFrame
    """
    period_map = {
        "近一月": "30",
        "近三月": "90",
        "近六月": "180",
        "近一年": "360",
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "D5_BUYER_NUM,D1_AVERAGE_INCREASE",
        "sortTypes": "-1,-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_BLOCKTRADE_OPERATEDEPT_RANK",
        "columns": "OPERATEDEPT_CODE,OPERATEDEPT_NAME,D1_BUYER_NUM,D1_AVERAGE_INCREASE,"
        "D1_RISE_PROBABILITY,D5_BUYER_NUM,D5_AVERAGE_INCREASE,D5_RISE_PROBABILITY,"
        "D10_BUYER_NUM,D10_AVERAGE_INCREASE,D10_RISE_PROBABILITY,D20_BUYER_NUM,"
        "D20_AVERAGE_INCREASE,D20_RISE_PROBABILITY,N_DATE,RELATED_ORG_CODE",
        "source": "WEB",
        "client": "WEB",
        "filter": f"(N_DATE=-{period_map[symbol]})",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in range(1, int(total_page) + 1):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "_",
        "营业部名称",
        "上榜后1天-买入次数",
        "上榜后1天-平均涨幅",
        "上榜后1天-上涨概率",
        "上榜后5天-买入次数",
        "上榜后5天-平均涨幅",
        "上榜后5天-上涨概率",
        "上榜后10天-买入次数",
        "上榜后10天-平均涨幅",
        "上榜后10天-上涨概率",
        "上榜后20天-买入次数",
        "上榜后20天-平均涨幅",
        "上榜后20天-上涨概率",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "营业部名称",
            "上榜后1天-买入次数",
            "上榜后1天-平均涨幅",
            "上榜后1天-上涨概率",
            "上榜后5天-买入次数",
            "上榜后5天-平均涨幅",
            "上榜后5天-上涨概率",
            "上榜后10天-买入次数",
            "上榜后10天-平均涨幅",
            "上榜后10天-上涨概率",
            "上榜后20天-买入次数",
            "上榜后20天-平均涨幅",
            "上榜后20天-上涨概率",
        ]
    ]
    big_df["上榜后1天-买入次数"] = pd.to_numeric(
        big_df["上榜后1天-买入次数"], errors="coerce"
    )
    big_df["上榜后1天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后1天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后1天-上涨概率"] = pd.to_numeric(
        big_df["上榜后1天-上涨概率"], errors="coerce"
    )
    big_df["上榜后5天-买入次数"] = pd.to_numeric(
        big_df["上榜后5天-买入次数"], errors="coerce"
    )
    big_df["上榜后5天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后5天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后5天-上涨概率"] = pd.to_numeric(
        big_df["上榜后5天-上涨概率"], errors="coerce"
    )
    big_df["上榜后10天-买入次数"] = pd.to_numeric(
        big_df["上榜后10天-买入次数"], errors="coerce"
    )
    big_df["上榜后10天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后10天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后10天-上涨概率"] = pd.to_numeric(
        big_df["上榜后10天-上涨概率"], errors="coerce"
    )
    big_df["上榜后20天-买入次数"] = pd.to_numeric(
        big_df["上榜后20天-买入次数"], errors="coerce"
    )
    big_df["上榜后20天-平均涨幅"] = pd.to_numeric(
        big_df["上榜后20天-平均涨幅"], errors="coerce"
    )
    big_df["上榜后20天-上涨概率"] = pd.to_numeric(
        big_df["上榜后20天-上涨概率"], errors="coerce"
    )
    return big_df


if __name__ == "__main__":
    stock_dzjy_sctj_df = stock_dzjy_sctj()
    print(stock_dzjy_sctj_df)

    stock_dzjy_mrmx_df = stock_dzjy_mrmx(
        symbol="债券", start_date="20220104", end_date="20220104"
    )
    print(stock_dzjy_mrmx_df)

    stock_dzjy_mrtj_df = stock_dzjy_mrtj(start_date="20220105", end_date="20220105")
    print(stock_dzjy_mrtj_df)

    stock_dzjy_hygtj_df = stock_dzjy_hygtj(symbol="近三月")
    print(stock_dzjy_hygtj_df)

    stock_dzjy_hyyybtj_df = stock_dzjy_hyyybtj(symbol="近3日")
    print(stock_dzjy_hyyybtj_df)

    stock_dzjy_yybph_df = stock_dzjy_yybph(symbol="近三月")
    print(stock_dzjy_yybph_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/10/30 15:00
Desc: 全部A股-等权重市盈率、中位数市盈率
https://www.legulegu.com/stockdata/a-ttm-lyr
"""

import pandas as pd
import requests

from akshare.stock_feature.stock_a_indicator import get_token_lg, get_cookie_csrf


def stock_a_ttm_lyr() -> pd.DataFrame:
    """
    全部 A 股-等权重市盈率、中位数市盈率
    :return: 全部A股-等权重市盈率、中位数市盈率
    :rtype: pandas.DataFrame
    """

    url = "https://legulegu.com/api/stock-data/market-ttm-lyr"
    params = {
        "marketId": "5",
        "token": get_token_lg(),
    }
    # 获取 cookie 和 headers
    csrf_data = get_cookie_csrf(url="https://www.legulegu.com/stockdata/a-ttm-lyr")
    # 使用返回的 headers（已经是副本）
    request_headers = csrf_data["headers"].copy()
    request_headers.update(
        {
            "host": "www.legulegu.com",
            "referer": "https://www.legulegu.com/stockdata/a-ttm-lyr",
        }
    )
    # 使用独立的 session
    session = requests.Session()
    r = session.get(
        url,
        params=params,
        cookies=csrf_data["cookies"],
        headers=request_headers,
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    # 关闭 session
    session.close()
    return temp_df


if __name__ == "__main__":
    stock_a_ttm_lyr_df = stock_a_ttm_lyr()
    print(stock_a_ttm_lyr_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2019/10/20 10:21
Desc: 债券配置文件
"""

# bond-cov-sina
zh_sina_bond_hs_cov_url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeDataSimple"
zh_sina_bond_hs_cov_count_url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeStockCountSimple"
zh_sina_bond_hs_cov_hist_url = (
    "https://finance.sina.com.cn/realstock/company/{}/hisdata/klc_kl.js?d={}"
)
zh_sina_bond_hs_cov_payload = {
    "page": "1",
    "num": "80",
    "sort": "symbol",
    "asc": "1",
    "node": "hskzz_z",
    "_s_r_a": "page",
}

# bond-sina
zh_sina_bond_hs_url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData"
zh_sina_bond_hs_count_url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeStockCountSimple"
zh_sina_bond_hs_hist_url = (
    "https://finance.sina.com.cn/realstock/company/{}/hisdata/klc_kl.js?d={}"
)
zh_sina_bond_hs_payload = {
    "page": "1",
    "num": "80",
    "sort": "symbol",
    "asc": "1",
    "node": "hs_z",
    "_s_r_a": "page",
}

# headers
SHORT_HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.91 Safari/537.36"
}

# quote
MARKET_QUOTE_URL = "http://www.chinamoney.com.cn/ags/ms/cm-u-md-bond/CbMktMakQuot?flag=1&lang=cn&abdAssetEncdShrtDesc=&emaEntyEncdShrtDesc="
MARKET_QUOTE_PAYLOAD = {
    "flag": "1",
    "lang": "cn",
    "abdAssetEncdShrtDesc": "",
    "emaEntyEncdShrtDesc": "",
}

# trade
MARKET_TRADE_URL = (
    "http://www.chinamoney.com.cn/ags/ms/cm-u-md-bond/CbtPri?lang=cn&flag=1&bondName="
)
MARKET_TRADE_PAYLOAD = {"lang": "cn", "flag": "1", "bondName": ""}



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/2/5 18:00
Desc: 东方财富网-数据中心-沪深港通-市场概括
https://data.eastmoney.com/hsgt/hsgtDetail/scgk.html
"""

import pandas as pd
import requests


def stock_hsgt_fund_min_em(symbol: str = "北向资金") -> pd.DataFrame:
    """
    东方财富-数据中心-沪深港通-市场概括-分时数据
    https://data.eastmoney.com/hsgt/hsgtDetail/scgk.html
    :param symbol: 北向资金; choice of {"北向资金", "南向资金"}
    :type symbol: str
    :return: 沪深港通持股-分时数据
    :rtype: pandas.DataFrame
    """
    url = "https://push2.eastmoney.com/api/qt/kamtbs.rtmin/get"
    params = {
        "fields1": "f1,f2,f3,f4",
        "fields2": "f51,f54,f52,f58,f53,f62,f56,f57,f60,f61",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "_": "1707125786160",
    }
    r = requests.get(url, params=params)
    data_json = r.json()

    if symbol == "南向资金":
        n2s_str_list = data_json["data"]["n2s"]
        temp_df = pd.DataFrame([item.split(",") for item in n2s_str_list])
        temp_df["date"] = data_json["data"]["n2sDate"]
        temp_df = temp_df.iloc[:, [0, 1, 3, 5, -1]]
        temp_df.columns = ["时间", "港股通(沪)", "港股通(深)", "南向资金", "日期"]
        temp_df = temp_df[["日期", "时间", "港股通(沪)", "港股通(深)", "南向资金"]]
        temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
        temp_df["港股通(沪)"] = pd.to_numeric(temp_df["港股通(沪)"], errors="coerce")
        temp_df["港股通(深)"] = pd.to_numeric(temp_df["港股通(深)"], errors="coerce")
        temp_df["南向资金"] = pd.to_numeric(temp_df["南向资金"], errors="coerce")
        return temp_df
    else:
        s2n_str_list = data_json["data"]["s2n"]
        temp_df = pd.DataFrame([item.split(",") for item in s2n_str_list])
        temp_df["date"] = data_json["data"]["s2nDate"]
        temp_df = temp_df.iloc[:, [0, 1, 3, 5, -1]]
        temp_df.columns = ["时间", "沪股通", "深股通", "北向资金", "日期"]
        temp_df = temp_df[["日期", "时间", "沪股通", "深股通", "北向资金"]]
        temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
        temp_df["沪股通"] = pd.to_numeric(temp_df["沪股通"], errors="coerce")
        temp_df["深股通"] = pd.to_numeric(temp_df["深股通"], errors="coerce")
        temp_df["北向资金"] = pd.to_numeric(temp_df["北向资金"], errors="coerce")
        return temp_df


if __name__ == "__main__":
    stock_hsgt_fund_min_em_df = stock_hsgt_fund_min_em(symbol="北向资金")
    print(stock_hsgt_fund_min_em_df)

    stock_hsgt_fund_min_em_df = stock_hsgt_fund_min_em(symbol="南向资金")
    print(stock_hsgt_fund_min_em_df)
